# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: device/heat_pump/ble/v1/wifi_network.proto
# Protobuf Python Version: 6.30.0
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    30,
    0,
    '',
    'device/heat_pump/ble/v1/wifi_network.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n*device/heat_pump/ble/v1/wifi_network.proto\x12\x17\x64\x65vice.heat_pump.ble.v1\"d\n\x0bWifiNetwork\x12\x13\n\x0bmac_address\x18\x01 \x01(\t\x12\x0c\n\x04ssid\x18\x02 \x01(\t\x12\x17\n\x0fsignal_strength\x18\x03 \x01(\x05\x12\x19\n\x11password_required\x18\x04 \x01(\x08\x42\xa6\x01\n\x17\x64\x65vice.heat_pump.ble.v1B\x10WifiNetworkProtoP\x01\xa2\x02\x03\x44HB\xaa\x02\x16\x44\x65vice.HeatPump.Ble.V1\xca\x02\x16\x44\x65vice\\HeatPump\\Ble\\V1\xe2\x02\"Device\\HeatPump\\Ble\\V1\\GPBMetadata\xea\x02\x19\x44\x65vice::HeatPump::Ble::V1b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'device.heat_pump.ble.v1.wifi_network_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\027device.heat_pump.ble.v1B\020WifiNetworkProtoP\001\242\002\003DHB\252\002\026Device.HeatPump.Ble.V1\312\002\026Device\\HeatPump\\Ble\\V1\342\002\"Device\\HeatPump\\Ble\\V1\\GPBMetadata\352\002\031Device::HeatPump::Ble::V1'
  _globals['_WIFINETWORK']._serialized_start=71
  _globals['_WIFINETWORK']._serialized_end=171
# @@protoc_insertion_point(module_scope)


from .airahome import AiraHome

# Version is automatically set by setuptools-scm from git tags
try:
    from ._version import version as __version__
except ImportError:
    # Fallback for development installs without _version.py
    from importlib.metadata import version
    __version__ = version("pyairahome")

__all__ = ['AiraHome']


# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: schedule/v1/rrule.proto
# Protobuf Python Version: 6.30.0
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    30,
    0,
    '',
    'schedule/v1/rrule.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x17schedule/v1/rrule.proto\x12\x0bschedule.v1\x1a\x1fgoogle/protobuf/timestamp.proto\"9\n\x03Nth\x12\x0b\n\x03nth\x18\x01 \x01(\x05\x12%\n\x07weekday\x18\x02 \x01(\x0e\x32\x14.schedule.v1.Weekday\".\n\x05\x45very\x12%\n\x07weekday\x18\x01 \x01(\x0e\x32\x14.schedule.v1.Weekday\"[\n\tByWeekday\x12#\n\x05\x65very\x18\x01 \x01(\x0b\x32\x12.schedule.v1.EveryH\x00\x12\x1f\n\x03nth\x18\x02 \x01(\x0b\x32\x10.schedule.v1.NthH\x00\x42\x08\n\x06group0\"f\n\x08RRuleEnd\x12\x0f\n\x05\x63ount\x18\x01 \x01(\rH\x00\x12+\n\x05until\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.TimestampH\x00\x12\x12\n\x08until_dt\x18\x03 \x01(\tH\x00\x42\x08\n\x06group0\"\xc0\x01\n\x05RRule\x12)\n\tfrequency\x18\x01 \x01(\x0e\x32\x16.schedule.v1.Frequency\x12\x10\n\x08interval\x18\x02 \x01(\r\x12(\n\trrule_end\x18\x03 \x01(\x0b\x32\x15.schedule.v1.RRuleEnd\x12$\n\x08\x62y_month\x18\x04 \x03(\x0e\x32\x12.schedule.v1.Month\x12*\n\nby_weekday\x18\x05 \x03(\x0b\x32\x16.schedule.v1.ByWeekday*\xb6\x01\n\x07Weekday\x12\x17\n\x13WEEKDAY_UNSPECIFIED\x10\x00\x12\x12\n\x0eWEEKDAY_MONDAY\x10\x01\x12\x13\n\x0fWEEKDAY_TUESDAY\x10\x02\x12\x15\n\x11WEEKDAY_WEDNESDAY\x10\x03\x12\x14\n\x10WEEKDAY_THURSDAY\x10\x04\x12\x12\n\x0eWEEKDAY_FRIDAY\x10\x05\x12\x14\n\x10WEEKDAY_SATURDAY\x10\x06\x12\x12\n\x0eWEEKDAY_SUNDAY\x10\x07*\xf8\x01\n\x05Month\x12\x15\n\x11MONTH_UNSPECIFIED\x10\x00\x12\x11\n\rMONTH_JANUARY\x10\x01\x12\x12\n\x0eMONTH_FEBRUARY\x10\x02\x12\x0f\n\x0bMONTH_MARCH\x10\x03\x12\x0f\n\x0bMONTH_APRIL\x10\x04\x12\r\n\tMONTH_MAY\x10\x05\x12\x0e\n\nMONTH_JUNE\x10\x06\x12\x0e\n\nMONTH_JULY\x10\x07\x12\x10\n\x0cMONTH_AUGUST\x10\x08\x12\x13\n\x0fMONTH_SEPTEMBER\x10\t\x12\x11\n\rMONTH_OCTOBER\x10\n\x12\x12\n\x0eMONTH_NOVEMBER\x10\x0b\x12\x12\n\x0eMONTH_DECEMBER\x10\x0c*\xac\x01\n\tFrequency\x12\x19\n\x15\x46REQUENCY_UNSPECIFIED\x10\x00\x12\x14\n\x10\x46REQUENCY_YEARLY\x10\x01\x12\x15\n\x11\x46REQUENCY_MONTHLY\x10\x02\x12\x14\n\x10\x46REQUENCY_WEEKLY\x10\x03\x12\x13\n\x0f\x46REQUENCY_DAILY\x10\x04\x12\x14\n\x10\x46REQUENCY_HOURLY\x10\x05\x12\x16\n\x12\x46REQUENCY_MINUTELY\x10\x06\x42\x66\n\x0bschedule.v1B\nRruleProtoP\x01\xa2\x02\x03SXX\xaa\x02\x0bSchedule.V1\xca\x02\x0bSchedule\\V1\xe2\x02\x17Schedule\\V1\\GPBMetadata\xea\x02\x0cSchedule::V1b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'schedule.v1.rrule_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\013schedule.v1B\nRruleProtoP\001\242\002\003SXX\252\002\013Schedule.V1\312\002\013Schedule\\V1\342\002\027Schedule\\V1\\GPBMetadata\352\002\014Schedule::V1'
  _globals['_WEEKDAY']._serialized_start=573
  _globals['_WEEKDAY']._serialized_end=755
  _globals['_MONTH']._serialized_start=758
  _globals['_MONTH']._serialized_end=1006
  _globals['_FREQUENCY']._serialized_start=1009
  _globals['_FREQUENCY']._serialized_end=1181
  _globals['_NTH']._serialized_start=73
  _globals['_NTH']._serialized_end=130
  _globals['_EVERY']._serialized_start=132
  _globals['_EVERY']._serialized_end=178
  _globals['_BYWEEKDAY']._serialized_start=180
  _globals['_BYWEEKDAY']._serialized_end=271
  _globals['_RRULEEND']._serialized_start=273
  _globals['_RRULEEND']._serialized_end=375
  _globals['_RRULE']._serialized_start=378
  _globals['_RRULE']._serialized_end=570
# @@protoc_insertion_point(module_scope)


# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc
import warnings

from . import user_invitation_pb2 as user__account_dot_v2_dot_user__invitation__pb2

GRPC_GENERATED_VERSION = '1.72.0'
GRPC_VERSION = grpc.__version__
_version_not_supported = False

try:
    from grpc._utilities import first_version_is_lower
    _version_not_supported = first_version_is_lower(GRPC_VERSION, GRPC_GENERATED_VERSION)
except ImportError:
    _version_not_supported = True

if _version_not_supported:
    raise RuntimeError(
        f'The grpc package installed is at version {GRPC_VERSION},'
        + f' but the generated code in user_account/v2/user_invitation_pb2_grpc.py depends on'
        + f' grpcio>={GRPC_GENERATED_VERSION}.'
        + f' Please upgrade your grpc module to grpcio>={GRPC_GENERATED_VERSION}'
        + f' or downgrade your generated code using grpcio-tools<={GRPC_VERSION}.'
    )


class UserInvitationServiceStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.DeleteInvitation = channel.unary_unary(
                '/user_account.v2.UserInvitationService/DeleteInvitation',
                request_serializer=user__account_dot_v2_dot_user__invitation__pb2.DeleteInvitationRequest.SerializeToString,
                response_deserializer=user__account_dot_v2_dot_user__invitation__pb2.DeleteInvitationResponse.FromString,
                _registered_method=True)
        self.InviteUserToHousehold = channel.unary_unary(
                '/user_account.v2.UserInvitationService/InviteUserToHousehold',
                request_serializer=user__account_dot_v2_dot_user__invitation__pb2.InviteUserToHouseholdRequest.SerializeToString,
                response_deserializer=user__account_dot_v2_dot_user__invitation__pb2.InviteUserToHouseholdResponse.FromString,
                _registered_method=True)
        self.AcceptInvitation = channel.unary_unary(
                '/user_account.v2.UserInvitationService/AcceptInvitation',
                request_serializer=user__account_dot_v2_dot_user__invitation__pb2.AcceptInvitationRequest.SerializeToString,
                response_deserializer=user__account_dot_v2_dot_user__invitation__pb2.AcceptInvitationResponse.FromString,
                _registered_method=True)
        self.GetInvitationsByHouseholdId = channel.unary_unary(
                '/user_account.v2.UserInvitationService/GetInvitationsByHouseholdId',
                request_serializer=user__account_dot_v2_dot_user__invitation__pb2.GetInvitationsByHouseholdIdRequest.SerializeToString,
                response_deserializer=user__account_dot_v2_dot_user__invitation__pb2.GetInvitationsByHouseholdIdResponse.FromString,
                _registered_method=True)


class UserInvitationServiceServicer(object):
    """Missing associated documentation comment in .proto file."""

    def DeleteInvitation(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def InviteUserToHousehold(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def AcceptInvitation(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetInvitationsByHouseholdId(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_UserInvitationServiceServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'DeleteInvitation': grpc.unary_unary_rpc_method_handler(
                    servicer.DeleteInvitation,
                    request_deserializer=user__account_dot_v2_dot_user__invitation__pb2.DeleteInvitationRequest.FromString,
                    response_serializer=user__account_dot_v2_dot_user__invitation__pb2.DeleteInvitationResponse.SerializeToString,
            ),
            'InviteUserToHousehold': grpc.unary_unary_rpc_method_handler(
                    servicer.InviteUserToHousehold,
                    request_deserializer=user__account_dot_v2_dot_user__invitation__pb2.InviteUserToHouseholdRequest.FromString,
                    response_serializer=user__account_dot_v2_dot_user__invitation__pb2.InviteUserToHouseholdResponse.SerializeToString,
            ),
            'AcceptInvitation': grpc.unary_unary_rpc_method_handler(
                    servicer.AcceptInvitation,
                    request_deserializer=user__account_dot_v2_dot_user__invitation__pb2.AcceptInvitationRequest.FromString,
                    response_serializer=user__account_dot_v2_dot_user__invitation__pb2.AcceptInvitationResponse.SerializeToString,
            ),
            'GetInvitationsByHouseholdId': grpc.unary_unary_rpc_method_handler(
                    servicer.GetInvitationsByHouseholdId,
                    request_deserializer=user__account_dot_v2_dot_user__invitation__pb2.GetInvitationsByHouseholdIdRequest.FromString,
                    response_serializer=user__account_dot_v2_dot_user__invitation__pb2.GetInvitationsByHouseholdIdResponse.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'user_account.v2.UserInvitationService', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))
    server.add_registered_method_handlers('user_account.v2.UserInvitationService', rpc_method_handlers)


 # This class is part of an EXPERIMENTAL API.
class UserInvitationService(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def DeleteInvitation(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/user_account.v2.UserInvitationService/DeleteInvitation',
            user__account_dot_v2_dot_user__invitation__pb2.DeleteInvitationRequest.SerializeToString,
            user__account_dot_v2_dot_user__invitation__pb2.DeleteInvitationResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def InviteUserToHousehold(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/user_account.v2.UserInvitationService/InviteUserToHousehold',
            user__account_dot_v2_dot_user__invitation__pb2.InviteUserToHouseholdRequest.SerializeToString,
            user__account_dot_v2_dot_user__invitation__pb2.InviteUserToHouseholdResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def AcceptInvitation(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/user_account.v2.UserInvitationService/AcceptInvitation',
            user__account_dot_v2_dot_user__invitation__pb2.AcceptInvitationRequest.SerializeToString,
            user__account_dot_v2_dot_user__invitation__pb2.AcceptInvitationResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def GetInvitationsByHouseholdId(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/user_account.v2.UserInvitationService/GetInvitationsByHouseholdId',
            user__account_dot_v2_dot_user__invitation__pb2.GetInvitationsByHouseholdIdRequest.SerializeToString,
            user__account_dot_v2_dot_user__invitation__pb2.GetInvitationsByHouseholdIdResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)


# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: device/power/controller/v1/current_transformer.proto
# Protobuf Python Version: 6.30.0
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    30,
    0,
    '',
    'device/power/controller/v1/current_transformer.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n4device/power/controller/v1/current_transformer.proto\x12\x1a\x64\x65vice.power.controller.v1\"\x90\x01\n\x0cThreePhaseCt\x12\x30\n\x04port\x18\x01 \x01(\x0e\x32\".device.power.controller.v1.CtPort\x12N\n\x0borientation\x18\x03 \x01(\x0e\x32\x39.device.power.controller.v1.CurrentTransformerOrientation\"\xbd\x01\n\rThreePhaseCts\x12\x38\n\x06phase1\x18\x01 \x01(\x0b\x32(.device.power.controller.v1.ThreePhaseCt\x12\x38\n\x06phase2\x18\x02 \x01(\x0b\x32(.device.power.controller.v1.ThreePhaseCt\x12\x38\n\x06phase3\x18\x03 \x01(\x0b\x32(.device.power.controller.v1.ThreePhaseCt\"\\\n\nOnePhaseCt\x12N\n\x0borientation\x18\x02 \x01(\x0e\x32\x39.device.power.controller.v1.CurrentTransformerOrientation\"\xaa\x01\n\x1f\x43urrentTransformerConfiguration\x12;\n\tone_phase\x18\x01 \x01(\x0b\x32&.device.power.controller.v1.OnePhaseCtH\x00\x12@\n\x0bthree_phase\x18\x02 \x01(\x0b\x32).device.power.controller.v1.ThreePhaseCtsH\x00\x42\x08\n\x06group0\"\xbc\x01\n\x17\x43urrentTransformerState\x12;\n\nscan_state\x18\x01 \x01(\x0e\x32\'.device.power.controller.v1.CtScanState\x12\x10\n\x08\x63t_ratio\x18\x02 \x01(\r\x12R\n\rconfiguration\x18\x03 \x01(\x0b\x32;.device.power.controller.v1.CurrentTransformerConfiguration*h\n\x0b\x43tScanState\x12\x1d\n\x19\x43T_SCAN_STATE_UNSPECIFIED\x10\x00\x12\x1e\n\x1a\x43T_SCAN_STATE_NOT_SCANNING\x10\x01\x12\x1a\n\x16\x43T_SCAN_STATE_SCANNING\x10\x02*\xa5\x01\n\x1d\x43urrentTransformerOrientation\x12/\n+CURRENT_TRANSFORMER_ORIENTATION_UNSPECIFIED\x10\x00\x12(\n$CURRENT_TRANSFORMER_ORIENTATION_LEFT\x10\x01\x12)\n%CURRENT_TRANSFORMER_ORIENTATION_RIGHT\x10\x02*N\n\x06\x43tPort\x12\x17\n\x13\x43T_PORT_UNSPECIFIED\x10\x00\x12\r\n\tCT_PORT_1\x10\x01\x12\r\n\tCT_PORT_2\x10\x02\x12\r\n\tCT_PORT_3\x10\x03\x42\xc0\x01\n\x1a\x64\x65vice.power.controller.v1B\x17\x43urrentTransformerProtoP\x01\xa2\x02\x03\x44PC\xaa\x02\x1a\x44\x65vice.Power.Controller.V1\xca\x02\x1a\x44\x65vice\\Power\\Controller\\V1\xe2\x02&Device\\Power\\Controller\\V1\\GPBMetadata\xea\x02\x1d\x44\x65vice::Power::Controller::V1b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'device.power.controller.v1.current_transformer_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\032device.power.controller.v1B\027CurrentTransformerProtoP\001\242\002\003DPC\252\002\032Device.Power.Controller.V1\312\002\032Device\\Power\\Controller\\V1\342\002&Device\\Power\\Controller\\V1\\GPBMetadata\352\002\035Device::Power::Controller::V1'
  _globals['_CTSCANSTATE']._serialized_start=881
  _globals['_CTSCANSTATE']._serialized_end=985
  _globals['_CURRENTTRANSFORMERORIENTATION']._serialized_start=988
  _globals['_CURRENTTRANSFORMERORIENTATION']._serialized_end=1153
  _globals['_CTPORT']._serialized_start=1155
  _globals['_CTPORT']._serialized_end=1233
  _globals['_THREEPHASECT']._serialized_start=85
  _globals['_THREEPHASECT']._serialized_end=229
  _globals['_THREEPHASECTS']._serialized_start=232
  _globals['_THREEPHASECTS']._serialized_end=421
  _globals['_ONEPHASECT']._serialized_start=423
  _globals['_ONEPHASECT']._serialized_end=515
  _globals['_CURRENTTRANSFORMERCONFIGURATION']._serialized_start=518
  _globals['_CURRENTTRANSFORMERCONFIGURATION']._serialized_end=688
  _globals['_CURRENTTRANSFORMERSTATE']._serialized_start=691
  _globals['_CURRENTTRANSFORMERSTATE']._serialized_end=879
# @@protoc_insertion_point(module_scope)


# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: util/v1/type_of_day.proto
# Protobuf Python Version: 6.30.0
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    30,
    0,
    '',
    'util/v1/type_of_day.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x19util/v1/type_of_day.proto\x12\x07util.v1*Z\n\tTypeOfDay\x12\x1b\n\x17TYPE_OF_DAY_UNSPECIFIED\x10\x00\x12\x17\n\x13TYPE_OF_DAY_WEEKDAY\x10\x01\x12\x17\n\x13TYPE_OF_DAY_WEEKEND\x10\x02\x42V\n\x07util.v1B\x0eTypeOfDayProtoP\x01\xa2\x02\x03UXX\xaa\x02\x07Util.V1\xca\x02\x07Util\\V1\xe2\x02\x13Util\\V1\\GPBMetadata\xea\x02\x08Util::V1b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'util.v1.type_of_day_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\007util.v1B\016TypeOfDayProtoP\001\242\002\003UXX\252\002\007Util.V1\312\002\007Util\\V1\342\002\023Util\\V1\\GPBMetadata\352\002\010Util::V1'
  _globals['_TYPEOFDAY']._serialized_start=38
  _globals['_TYPEOFDAY']._serialized_end=128
# @@protoc_insertion_point(module_scope)


# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: device/heat_pump/command/v1/set_room_temp_setpoint_delta.proto
# Protobuf Python Version: 6.30.0
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    30,
    0,
    '',
    'device/heat_pump/command/v1/set_room_temp_setpoint_delta.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import duration_pb2 as google_dot_protobuf_dot_duration__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n>device/heat_pump/command/v1/set_room_temp_setpoint_delta.proto\x12\x1b\x64\x65vice.heat_pump.command.v1\x1a\x1egoogle/protobuf/duration.proto\"q\n\x18SetRoomTempSetpointDelta\x12U\n\x19room_temp_setpoint_deltas\x18\x01 \x03(\x0b\x32\x32.device.heat_pump.command.v1.RoomTempSetpointDelta\"\xe5\x01\n\x15RoomTempSetpointDelta\x12\x45\n\x04kind\x18\x01 \x01(\x0e\x32\x37.device.heat_pump.command.v1.RoomTempSetpointDelta.Kind\x12+\n\x08\x64uration\x18\x02 \x01(\x0b\x32\x19.google.protobuf.Duration\x12\x13\n\x0b\x64\x65lta_zone1\x18\x03 \x01(\x02\x12\x13\n\x0b\x64\x65lta_zone2\x18\x04 \x01(\x02\".\n\x04Kind\x12\x14\n\x10KIND_UNSPECIFIED\x10\x00\x12\x10\n\x0cKIND_HEATING\x10\x01*\x82\x04\n\x1dSetRoomTempSetpointDeltaError\x12\x32\n.SET_ROOM_TEMP_SETPOINT_DELTA_ERROR_UNSPECIFIED\x10\x00\x12>\n:SET_ROOM_TEMP_SETPOINT_DELTA_ERROR_MISSING_SETPOINT_DELTAS\x10\x01\x12>\n:SET_ROOM_TEMP_SETPOINT_DELTA_ERROR_TOO_LONG_TOTAL_DURATION\x10\x02\x12\x37\n3SET_ROOM_TEMP_SETPOINT_DELTA_ERROR_INVALID_DURATION\x10\x03\x12@\n<SET_ROOM_TEMP_SETPOINT_DELTA_ERROR_DELTA_MAGNITUDE_TOO_LARGE\x10\x04\x12I\nESET_ROOM_TEMP_SETPOINT_DELTA_ERROR_MISSING_SETPOINT_DELTA_TEMPERATURE\x10\x05\x12\x33\n/SET_ROOM_TEMP_SETPOINT_DELTA_ERROR_MISSING_KIND\x10\x06\x12\x32\n.SET_ROOM_TEMP_SETPOINT_DELTA_ERROR_UNSUPPORTED\x10\x07\x42\xc7\x01\n\x1b\x64\x65vice.heat_pump.command.v1B\x1dSetRoomTempSetpointDeltaProtoP\x01\xa2\x02\x03\x44HC\xaa\x02\x1a\x44\x65vice.HeatPump.Command.V1\xca\x02\x1a\x44\x65vice\\HeatPump\\Command\\V1\xe2\x02&Device\\HeatPump\\Command\\V1\\GPBMetadata\xea\x02\x1d\x44\x65vice::HeatPump::Command::V1b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'device.heat_pump.command.v1.set_room_temp_setpoint_delta_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\033device.heat_pump.command.v1B\035SetRoomTempSetpointDeltaProtoP\001\242\002\003DHC\252\002\032Device.HeatPump.Command.V1\312\002\032Device\\HeatPump\\Command\\V1\342\002&Device\\HeatPump\\Command\\V1\\GPBMetadata\352\002\035Device::HeatPump::Command::V1'
  _globals['_SETROOMTEMPSETPOINTDELTAERROR']._serialized_start=475
  _globals['_SETROOMTEMPSETPOINTDELTAERROR']._serialized_end=989
  _globals['_SETROOMTEMPSETPOINTDELTA']._serialized_start=127
  _globals['_SETROOMTEMPSETPOINTDELTA']._serialized_end=240
  _globals['_ROOMTEMPSETPOINTDELTA']._serialized_start=243
  _globals['_ROOMTEMPSETPOINTDELTA']._serialized_end=472
  _globals['_ROOMTEMPSETPOINTDELTA_KIND']._serialized_start=426
  _globals['_ROOMTEMPSETPOINTDELTA_KIND']._serialized_end=472
# @@protoc_insertion_point(module_scope)


"""Simple class to login with cognito and load tokens in the TokenManager class."""
# auth/cognito_auth.py

from pycognito import Cognito
from .token_manager import TokenManager
from ..utils import AuthenticationError, NotLoggedInException

class CognitoAuth:
    def __init__(self,
                 user_pool_id :str,
                 client_id :str):
        """Initialize the CognitoAuth with user pool ID, client ID. Allows logging in and creating a TokenManager."""
        self.user_pool_id = user_pool_id
        self.client_id = client_id
        self._tokens = None

    def login_credentials(self, username: str, password: str):
        """Login using username and password. Returns a TokenManager instance"""	
        try:
            user = Cognito(self.user_pool_id, self.client_id, username=username)
            user.authenticate(password=password)
            self._tokens = TokenManager(
                user_pool_id=self.user_pool_id,
                client_id=self.client_id,
                id_token=user.id_token,
                access_token=user.access_token,
                refresh_token=user.refresh_token
            )
            self._tokens.verify_tokens()
            return self._tokens
        except Exception as e:
            raise AuthenticationError("Login failed while attempting to login with username and password") from e

    def login_tokens(self, id_token: str, access_token: str, refresh_token: str):
        """Login using existing tokens. Returns a TokenManager instance"""
        try:
            self._tokens = TokenManager(
                user_pool_id=self.user_pool_id,
                client_id=self.client_id,
                id_token=id_token,
                access_token=access_token,
                refresh_token=refresh_token
            )
            self._tokens.verify_tokens()
            return self._tokens
        except Exception as e:
            raise AuthenticationError("Login failed while attempting to login with tokens") from e

    def get_tokens(self):
        """Get the TokenManager instance if available."""
        if self._tokens is None:
            raise NotLoggedInException("Please login before doing anything else")
        self._tokens.refresh_tokens() # verify and refresh the tokens before using them
        return self._tokens




# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: device/heat_pump/state/v1/features.proto
# Protobuf Python Version: 6.30.0
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    30,
    0,
    '',
    'device/heat_pump/state/v1/features.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n(device/heat_pump/state/v1/features.proto\x12\x19\x64\x65vice.heat_pump.state.v1\"P\n\x08\x46\x65\x61tures\x12\x44\n\x10\x66\x65\x61ture_versions\x18\x01 \x03(\x0b\x32*.device.heat_pump.state.v1.FeatureVersions\"\x8d\x01\n\x0f\x46\x65\x61tureVersions\x12\x33\n\x07\x66\x65\x61ture\x18\x01 \x01(\x0e\x32\".device.heat_pump.state.v1.Feature\x12\x45\n\x12supported_versions\x18\x02 \x03(\x0e\x32).device.heat_pump.state.v1.FeatureVersion*\x83\x01\n\x07\x46\x65\x61ture\x12\x17\n\x13\x46\x45\x41TURE_UNSPECIFIED\x10\x00\x12\x16\n\x12\x46\x45\x41TURE_LEGIONELLA\x10\x01\x12\x13\n\x0f\x46\x45\x41TURE_HEATING\x10\x02\x12\x15\n\x11\x46\x45\x41TURE_SCHEDULER\x10\x03\x12\x1b\n\x17\x46\x45\x41TURE_STATE_REPORTING\x10\x04*\xbb\x01\n\x0e\x46\x65\x61tureVersion\x12\x1f\n\x1b\x46\x45\x41TURE_VERSION_UNSPECIFIED\x10\x00\x12\x15\n\x11\x46\x45\x41TURE_VERSION_1\x10\x01\x12\x15\n\x11\x46\x45\x41TURE_VERSION_2\x10\x02\x12\x15\n\x11\x46\x45\x41TURE_VERSION_3\x10\x03\x12\x15\n\x11\x46\x45\x41TURE_VERSION_4\x10\x04\x12\x15\n\x11\x46\x45\x41TURE_VERSION_5\x10\x05\x12\x15\n\x11\x46\x45\x41TURE_VERSION_6\x10\x06\x42\xad\x01\n\x19\x64\x65vice.heat_pump.state.v1B\rFeaturesProtoP\x01\xa2\x02\x03\x44HS\xaa\x02\x18\x44\x65vice.HeatPump.State.V1\xca\x02\x18\x44\x65vice\\HeatPump\\State\\V1\xe2\x02$Device\\HeatPump\\State\\V1\\GPBMetadata\xea\x02\x1b\x44\x65vice::HeatPump::State::V1b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'device.heat_pump.state.v1.features_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\031device.heat_pump.state.v1B\rFeaturesProtoP\001\242\002\003DHS\252\002\030Device.HeatPump.State.V1\312\002\030Device\\HeatPump\\State\\V1\342\002$Device\\HeatPump\\State\\V1\\GPBMetadata\352\002\033Device::HeatPump::State::V1'
  _globals['_FEATURE']._serialized_start=298
  _globals['_FEATURE']._serialized_end=429
  _globals['_FEATUREVERSION']._serialized_start=432
  _globals['_FEATUREVERSION']._serialized_end=619
  _globals['_FEATURES']._serialized_start=71
  _globals['_FEATURES']._serialized_end=151
  _globals['_FEATUREVERSIONS']._serialized_start=154
  _globals['_FEATUREVERSIONS']._serialized_end=295
# @@protoc_insertion_point(module_scope)


# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: device/power/grid/v1/grid_code_preset.proto
# Protobuf Python Version: 6.30.0
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    30,
    0,
    '',
    'device/power/grid/v1/grid_code_preset.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n+device/power/grid/v1/grid_code_preset.proto\x12\x14\x64\x65vice.power.grid.v1*O\n\x0eGridCodePreset\x12 \n\x1cGRID_CODE_PRESET_UNSPECIFIED\x10\x00\x12\x1b\n\x17GRID_CODE_PRESET_SWEDEN\x10\x01\x42\x9e\x01\n\x14\x64\x65vice.power.grid.v1B\x13GridCodePresetProtoP\x01\xa2\x02\x03\x44PG\xaa\x02\x14\x44\x65vice.Power.Grid.V1\xca\x02\x14\x44\x65vice\\Power\\Grid\\V1\xe2\x02 Device\\Power\\Grid\\V1\\GPBMetadata\xea\x02\x17\x44\x65vice::Power::Grid::V1b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'device.power.grid.v1.grid_code_preset_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\024device.power.grid.v1B\023GridCodePresetProtoP\001\242\002\003DPG\252\002\024Device.Power.Grid.V1\312\002\024Device\\Power\\Grid\\V1\342\002 Device\\Power\\Grid\\V1\\GPBMetadata\352\002\027Device::Power::Grid::V1'
  _globals['_GRIDCODEPRESET']._serialized_start=69
  _globals['_GRIDCODEPRESET']._serialized_end=148
# @@protoc_insertion_point(module_scope)


from ..device.heat_pump.statistics.v1.service_pb2 import Granularity
from ..device.heat_pump.ble.v1.get_data_pb2 import DataType
from enum import Enum

# Explicitly define __all__ for static analysis
__all__ = ['Granularity', 'GetDataType']

# Convert Protobuf Granularity to Python Enum
Granularity = Enum('Granularity', {
    name: descriptor.number
    for name, descriptor in Granularity.DESCRIPTOR.values_by_name.items()
})

GetDataType = Enum('GetDataType', {
    name: descriptor.number
    for name, descriptor in DataType.DESCRIPTOR.values_by_name.items()
})


from .exceptions import NotLoggedInException, AuthenticationError, UnknownTypeException, UnknownCommandException, TokenError, BLEDiscoveryError, BLEConnectionError, BLEInitializationError
from .commands import CommandUtils
from .utils import Utils

__all__ = ['Utils', 'NotLoggedInException', 'AuthenticationError', 'UnknownTypeException', 'UnknownCommandException', 'TokenError', 'BLEDiscoveryError', 'BLEConnectionError', 'BLEInitializationError', 'CommandUtils']


# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: device/heat_pump/command/v1/set_scheduled_heat_curve_deltas.proto
# Protobuf Python Version: 6.30.0
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    30,
    0,
    '',
    'device/heat_pump/command/v1/set_scheduled_heat_curve_deltas.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import duration_pb2 as google_dot_protobuf_dot_duration__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\nAdevice/heat_pump/command/v1/set_scheduled_heat_curve_deltas.proto\x12\x1b\x64\x65vice.heat_pump.command.v1\x1a\x1egoogle/protobuf/duration.proto\"U\n\x17ScheduledHeatCurveDelta\x12\n\n\x02p1\x18\x01 \x01(\x02\x12\n\n\x02p2\x18\x02 \x01(\x02\x12\n\n\x02p3\x18\x03 \x01(\x02\x12\n\n\x02p4\x18\x04 \x01(\x02\x12\n\n\x02p5\x18\x05 \x01(\x02\"\xd1\x01\n\x18ScheduledHeatCurveDeltas\x12+\n\x08\x64uration\x18\x01 \x01(\x0b\x32\x19.google.protobuf.Duration\x12\x43\n\x05zone1\x18\x02 \x01(\x0b\x32\x34.device.heat_pump.command.v1.ScheduledHeatCurveDelta\x12\x43\n\x05zone2\x18\x03 \x01(\x0b\x32\x34.device.heat_pump.command.v1.ScheduledHeatCurveDelta\"o\n\x1bSetScheduledHeatCurveDeltas\x12P\n\x11heat_curve_deltas\x18\x01 \x03(\x0b\x32\x35.device.heat_pump.command.v1.ScheduledHeatCurveDeltas*\xe2\x02\n\x16SetHeatCurveDeltaError\x12*\n&SET_HEAT_CURVE_DELTA_ERROR_UNSPECIFIED\x10\x00\x12\x36\n2SET_HEAT_CURVE_DELTA_ERROR_MISSING_SETPOINT_DELTAS\x10\x01\x12\x36\n2SET_HEAT_CURVE_DELTA_ERROR_TOO_LONG_TOTAL_DURATION\x10\x02\x12/\n+SET_HEAT_CURVE_DELTA_ERROR_INVALID_DURATION\x10\x03\x12\x38\n4SET_HEAT_CURVE_DELTA_ERROR_DELTA_MAGNITUDE_TOO_LARGE\x10\x04\x12\x41\n=SET_HEAT_CURVE_DELTA_ERROR_MISSING_SETPOINT_DELTA_TEMPERATURE\x10\x05\x42\xca\x01\n\x1b\x64\x65vice.heat_pump.command.v1B SetScheduledHeatCurveDeltasProtoP\x01\xa2\x02\x03\x44HC\xaa\x02\x1a\x44\x65vice.HeatPump.Command.V1\xca\x02\x1a\x44\x65vice\\HeatPump\\Command\\V1\xe2\x02&Device\\HeatPump\\Command\\V1\\GPBMetadata\xea\x02\x1d\x44\x65vice::HeatPump::Command::V1b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'device.heat_pump.command.v1.set_scheduled_heat_curve_deltas_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\033device.heat_pump.command.v1B SetScheduledHeatCurveDeltasProtoP\001\242\002\003DHC\252\002\032Device.HeatPump.Command.V1\312\002\032Device\\HeatPump\\Command\\V1\342\002&Device\\HeatPump\\Command\\V1\\GPBMetadata\352\002\035Device::HeatPump::Command::V1'
  _globals['_SETHEATCURVEDELTAERROR']._serialized_start=543
  _globals['_SETHEATCURVEDELTAERROR']._serialized_end=897
  _globals['_SCHEDULEDHEATCURVEDELTA']._serialized_start=130
  _globals['_SCHEDULEDHEATCURVEDELTA']._serialized_end=215
  _globals['_SCHEDULEDHEATCURVEDELTAS']._serialized_start=218
  _globals['_SCHEDULEDHEATCURVEDELTAS']._serialized_end=427
  _globals['_SETSCHEDULEDHEATCURVEDELTAS']._serialized_start=429
  _globals['_SETSCHEDULEDHEATCURVEDELTAS']._serialized_end=540
# @@protoc_insertion_point(module_scope)


"""Main class for the Aira Home library, providing high-level access to auth and heatpump data/controls."""
# airahome.py
from .device.heat_pump.command.v1 import command_pb2
from .utils import CommandUtils, BLEInitializationError
from .config import Settings
from .cloud import Cloud
from .ble import Ble


class AiraHome:
    def __init__(self,
                 user_pool_id: str = Settings.USER_POOL_IDS[0],
                 client_id: str = Settings.CLIENT_ID,
                 aira_backend: str = Settings.AIRA_BACKEND,
                 user_agent: str = Settings.USER_AGENT,
                 app_package: str = Settings.APP_PACKAGE,
                 app_version: str = Settings.APP_VERSION,
                 grpc_timeout: int = Settings.GRPC_TIMEOUT,
                 insecure_characteristic: str = Settings.INSECURE_CHARACTERISTIC,
                 secure_characteristic: str = Settings.SECURE_CHARACTERISTIC,
                 default_uuid_selection: int = Settings.DEFAULT_UUID_SELECTION,
                 ble_notify_timeout: int = Settings.BLE_NOTIFY_TIMEOUT,
                 max_ble_chunk_size: int = Settings.MAX_BLE_CHUNK_SIZE):
        # Store configuration for cloud access
        self.user_pool_id = user_pool_id
        self.client_id = client_id
        self.aira_backend = aira_backend
        self.user_agent = user_agent
        self.app_package = app_package
        self.app_version = app_version
        self.grpc_timeout = grpc_timeout
        # Store configuration for ble access
        self.insecure_characteristic = insecure_characteristic
        self.secure_characteristic = secure_characteristic
        self.default_uuid_selection = default_uuid_selection
        self.ble_notify_timeout = ble_notify_timeout
        self.max_ble_chunk_size = max_ble_chunk_size
        
        # Initialize cloud instance with reference to this class as parent
        self._cloud = None
        # Initialize ble instance with reference to this class as parent
        self._ble = None

        # Store data needed for simple ble usage
        self.certificate = None
        self.uuid = None

        # Utils
        self.command_list = self.get_command_list()

    @property
    def cloud(self):
        """Get the Cloud instance with access to parent AiraHome methods."""
        if self._cloud is None:
            self._cloud = Cloud(self)
        return self._cloud

    @property
    def ble(self):
        """Get the Ble instance with access to parent AiraHome methods."""
        if self._ble is None:
            self._ble = Ble(self)
        return self._ble

    ###
    # Internal/Helpers methods
    ###

    def get_command_list(self):
        """Get the list of available commands."""
        commands = []
        supported_commands = command_pb2.Command.DESCRIPTOR.fields_by_name.keys()
        for command in CommandUtils.find_in_modules(Settings.COMMAND_PACKAGE):
            if CommandUtils.camel_case_to_snake_case(command) in supported_commands:
                commands.append(command)
        return commands

    def get_command_fields(self, command: str, raw: bool = False):
        """Get the fields of a specific command."""
        return CommandUtils.get_message_field(command, Settings.COMMAND_PACKAGE, raw=raw)
    
    def init_ble(self) -> bool:
        """Initialize BLE by fetching the certificate and UUID from the cloud."""
        if not self.certificate or not self.uuid:
            devices = self.cloud.get_devices(raw=False)
            if len(devices["devices"])-1 < self.default_uuid_selection:
                raise BLEInitializationError(f"Default UUID selection index {self.default_uuid_selection} is out of range for available devices ({len(devices['devices'])}). Please adjust using default_uuid_selection parameter when initiating AiraHome class.")
             
            device = devices["devices"][self.default_uuid_selection]
            self.uuid = device["id"]["value"]

            device_details = self.cloud.get_device_details(self.uuid, raw=False)

            self.certificate = device_details["heat_pump"]["certificate"]["certificate_pem"]

            # try connecting
            return self.ble.connect()
        
        return False


# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: device/heat_pump/command/v1/command_error.proto
# Protobuf Python Version: 6.30.0
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    30,
    0,
    '',
    'device/heat_pump/command/v1/command_error.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from ...modbus.v1 import exception_pb2 as device_dot_heat__pump_dot_modbus_dot_v1_dot_exception__pb2
from . import set_telemetry_interval_pb2 as device_dot_heat__pump_dot_command_dot_v1_dot_set__telemetry__interval__pb2
from . import set_scheduled_heat_curve_deltas_pb2 as device_dot_heat__pump_dot_command_dot_v1_dot_set__scheduled__heat__curve__deltas__pb2
from . import set_room_temp_setpoint_delta_pb2 as device_dot_heat__pump_dot_command_dot_v1_dot_set__room__temp__setpoint__delta__pb2
from . import set_room_supply_setpoint_delta_pb2 as device_dot_heat__pump_dot_command_dot_v1_dot_set__room__supply__setpoint__delta__pb2
from . import set_dhw_setpoint_delta_pb2 as device_dot_heat__pump_dot_command_dot_v1_dot_set__dhw__setpoint__delta__pb2
from . import missing_field_error_pb2 as device_dot_heat__pump_dot_command_dot_v1_dot_missing__field__error__pb2
from . import database_error_pb2 as device_dot_heat__pump_dot_command_dot_v1_dot_database__error__pb2
from . import configure_heat_pump_pb2 as device_dot_heat__pump_dot_command_dot_v1_dot_configure__heat__pump__pb2
from . import ccv_error_pb2 as device_dot_heat__pump_dot_command_dot_v1_dot_ccv__error__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n/device/heat_pump/command/v1/command_error.proto\x12\x1b\x64\x65vice.heat_pump.command.v1\x1a*device/heat_pump/modbus/v1/exception.proto\x1a\x38\x64\x65vice/heat_pump/command/v1/set_telemetry_interval.proto\x1a\x41\x64\x65vice/heat_pump/command/v1/set_scheduled_heat_curve_deltas.proto\x1a>device/heat_pump/command/v1/set_room_temp_setpoint_delta.proto\x1a@device/heat_pump/command/v1/set_room_supply_setpoint_delta.proto\x1a\x38\x64\x65vice/heat_pump/command/v1/set_dhw_setpoint_delta.proto\x1a\x35\x64\x65vice/heat_pump/command/v1/missing_field_error.proto\x1a\x30\x64\x65vice/heat_pump/command/v1/database_error.proto\x1a\x35\x64\x65vice/heat_pump/command/v1/configure_heat_pump.proto\x1a+device/heat_pump/command/v1/ccv_error.proto\"\x91\x07\n\x0c\x43ommandError\x12\x0c\n\x04\x63ode\x18\x01 \x01(\x05\x12\x0f\n\x07message\x18\x02 \x01(\t\x12\x12\n\nwill_retry\x18\x08 \x01(\x08\x12?\n\x10modbus_exception\x18\t \x01(\x0e\x32%.device.heat_pump.modbus.v1.Exception\x12@\n\x0c\x63onfig_error\x18\x04 \x01(\x0b\x32(.device.heat_pump.command.v1.ConfigErrorH\x00\x12]\n\x1cset_dhw_setpoint_delta_error\x18\x05 \x01(\x0e\x32\x35.device.heat_pump.command.v1.SetDhwSetpointDeltaErrorH\x00\x12l\n$set_room_supply_setpoint_delta_error\x18\x06 \x01(\x0e\x32<.device.heat_pump.command.v1.SetRoomSupplySetpointDeltaErrorH\x00\x12:\n\tccv_error\x18\x07 \x01(\x0b\x32%.device.heat_pump.command.v1.CcvErrorH\x00\x12\x44\n\x0e\x64\x61tabase_error\x18\n \x01(\x0b\x32*.device.heat_pump.command.v1.DatabaseErrorH\x00\x12M\n\x13missing_field_error\x18\x0b \x01(\x0b\x32..device.heat_pump.command.v1.MissingFieldErrorH\x00\x12h\n\"set_room_temp_setpoint_delta_error\x18\x0c \x01(\x0e\x32:.device.heat_pump.command.v1.SetRoomTempSetpointDeltaErrorH\x00\x12Y\n\x1aset_heat_curve_delta_error\x18\r \x01(\x0e\x32\x33.device.heat_pump.command.v1.SetHeatCurveDeltaErrorH\x00\x12^\n\x1cset_telemetry_interval_error\x18\x0e \x01(\x0e\x32\x36.device.heat_pump.command.v1.SetTelemetryIntervalErrorH\x00\x42\x08\n\x06group0B\xbb\x01\n\x1b\x64\x65vice.heat_pump.command.v1B\x11\x43ommandErrorProtoP\x01\xa2\x02\x03\x44HC\xaa\x02\x1a\x44\x65vice.HeatPump.Command.V1\xca\x02\x1a\x44\x65vice\\HeatPump\\Command\\V1\xe2\x02&Device\\HeatPump\\Command\\V1\\GPBMetadata\xea\x02\x1d\x44\x65vice::HeatPump::Command::V1b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'device.heat_pump.command.v1.command_error_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\033device.heat_pump.command.v1B\021CommandErrorProtoP\001\242\002\003DHC\252\002\032Device.HeatPump.Command.V1\312\002\032Device\\HeatPump\\Command\\V1\342\002&Device\\HeatPump\\Command\\V1\\GPBMetadata\352\002\035Device::HeatPump::Command::V1'
  _globals['_COMMANDERROR']._serialized_start=643
  _globals['_COMMANDERROR']._serialized_end=1556
# @@protoc_insertion_point(module_scope)


"""Useful functions for the Aira Home library."""
# utils/utils.py
from google.protobuf.json_format import MessageToDict
from ..util.v1.local_date_time_pb2 import LocalDateTime
from google.protobuf.timestamp_pb2 import Timestamp
from google.protobuf.message import Message
from ..util.v1 import uuid_pb2 as uuid1_pb2
from ..util.v2 import uuid_pb2 as uuid2_pb2
from base64 import b64decode, b64encode
from .exceptions import UnknownTypeException
from datetime import datetime
from uuid import UUID


class Utils:
    @staticmethod
    def convert_to_dict(response: Message) -> dict:
        """Convert a protobuf response to a dictionary. Flattens heatpump id to v2 uuid format. Converts dates to datetime objects."""
        # assume heatpump_id to be v2. if the user uses v1 they should use raw and bypass this function
        # iterate every field and print type and content
        def replace_fields(response):
            # Convert the message to a dictionary for iteration
            response_python = MessageToDict(response, preserving_proto_field_name=True, always_print_fields_with_no_presence=True)

            def process_field(converted, original):
                if isinstance(converted, dict): # process dict
                    for key, value in converted.items():
                        if hasattr(original, key): # ensure the key exists in the original message
                            if isinstance(getattr(original, key), uuid1_pb2.Uuid): # if it's a UUID field
                                # Convert UUID to v2 format
                                converted[key]["value"] = Utils.convert_uuid_to_v2(value['value'])
                            elif isinstance(getattr(original, key), Timestamp): # if it's a Timestamp field
                                # Convert Timestamp to datetime
                                ts = getattr(original, key)
                                converted[key] = ts.ToDatetime()
                            elif isinstance(getattr(original, key), str): # if it's a string it could be date
                                try:
                                    dt = datetime.fromisoformat(value)
                                    converted[key] = dt
                                except ValueError:
                                    pass # not a date string, leave as is
                            else:
                                process_field(value, getattr(original, key)) # recurse into nested dict
                elif isinstance(converted, list):
                    for i in range(len(converted)):
                        process_field(converted[i], original[i])
        
            process_field(response_python, response)
            return response_python
        
        return replace_fields(response)
    
    @staticmethod
    def convert_to_uuid_list(device_ids):
        if isinstance(device_ids, list):
            if all(isinstance(device_id, uuid1_pb2.Uuid) for device_id in device_ids):
                heat_pump_ids = device_ids
            elif all(isinstance(device_id, str) for device_id in device_ids):
                heat_pump_ids = [uuid1_pb2.Uuid(value=b64decode(str.encode(device_id))) for device_id in device_ids]
            else:
                raise UnknownTypeException(f"Unknown type for {device_ids} list")
        elif isinstance(device_ids, uuid1_pb2.Uuid): # convert to list if given as single uuid
            heat_pump_ids = [device_ids]
        elif isinstance(device_ids, str): # convert to list of uuids if given in base64
            heat_pump_ids = [uuid1_pb2.Uuid(value=b64decode(str.encode(device_ids)))]
        else: # unknown type
            raise UnknownTypeException(f"Unknown type [{type(device_ids)}] for {device_ids}")

        return heat_pump_ids
    
    @staticmethod
    def convert_uuid_to_v2(device_id) -> str:
        """Convert a UUID to v2 format"""
        uuid = Utils.convert_to_uuid_list(device_id)[0]
        return str(UUID(uuid.value.hex()))
    
    @staticmethod
    def convert_uuid_from_v2(device_id: str) -> uuid1_pb2.Uuid:
        """Convert a v2 UUID to protobuf format"""
        return uuid1_pb2.Uuid(value=UUID(device_id).bytes)
    
    @staticmethod
    def convert_str_to_v2(device_id: str) -> str:
        """Convert a base64 UUID string to v2 format"""
        return uuid2_pb2.Uuid(value=device_id)
    
    @staticmethod
    def datetime_to_localdatetime(dt):
        return LocalDateTime(year=dt.year, month=dt.month, day=dt.day, hour=dt.hour, minute=dt.minute, second=dt.second, nanos=getattr(dt, "nanos", 0))
    
    @staticmethod
    def convert_to_timestamp(timestamp: float | int | None) -> Timestamp:
        if timestamp is None:
            _time = Timestamp()
            _time.GetCurrentTime()
        elif isinstance(timestamp, Timestamp):
            _time = timestamp
        elif isinstance(timestamp, int):
            _time = Timestamp(seconds=timestamp, nanos=0)
        elif isinstance(timestamp, float): # could have nanos wrong
            seconds, nanos = tuple(map(int, str(timestamp).split(".")))
            _time = Timestamp(seconds=seconds, nanos=nanos)
        elif isinstance(timestamp, datetime):
            seconds, nanos = tuple(map(int, str(timestamp.timestamp()).split(".")))
            _time = Timestamp(seconds=seconds, nanos=nanos)
        else:
            _time = Timestamp(seconds=0, nanos=0)
        return _time


# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: device/heat_pump/command/v1/disable_force_heating.proto
# Protobuf Python Version: 6.30.0
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    30,
    0,
    '',
    'device/heat_pump/command/v1/disable_force_heating.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n7device/heat_pump/command/v1/disable_force_heating.proto\x12\x1b\x64\x65vice.heat_pump.command.v1\"\x15\n\x13\x44isableForceHeatingB\xc2\x01\n\x1b\x64\x65vice.heat_pump.command.v1B\x18\x44isableForceHeatingProtoP\x01\xa2\x02\x03\x44HC\xaa\x02\x1a\x44\x65vice.HeatPump.Command.V1\xca\x02\x1a\x44\x65vice\\HeatPump\\Command\\V1\xe2\x02&Device\\HeatPump\\Command\\V1\\GPBMetadata\xea\x02\x1d\x44\x65vice::HeatPump::Command::V1b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'device.heat_pump.command.v1.disable_force_heating_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\033device.heat_pump.command.v1B\030DisableForceHeatingProtoP\001\242\002\003DHC\252\002\032Device.HeatPump.Command.V1\312\002\032Device\\HeatPump\\Command\\V1\342\002&Device\\HeatPump\\Command\\V1\\GPBMetadata\352\002\035Device::HeatPump::Command::V1'
  _globals['_DISABLEFORCEHEATING']._serialized_start=88
  _globals['_DISABLEFORCEHEATING']._serialized_end=109
# @@protoc_insertion_point(module_scope)


# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: device/heat_pump/ble/v1/system_check_state.proto
# Protobuf Python Version: 6.30.0
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    30,
    0,
    '',
    'device/heat_pump/ble/v1/system_check_state.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import duration_pb2 as google_dot_protobuf_dot_duration__pb2
from ...telemetry.v1.ccv.v1 import telemetry_data_pb2 as device_dot_heat__pump_dot_telemetry_dot_v1_dot_ccv_dot_v1_dot_telemetry__data__pb2
from ...state.v1 import state_pb2 as device_dot_heat__pump_dot_state_dot_v1_dot_state__pb2
from ...state.v1 import error_pb2 as device_dot_heat__pump_dot_state_dot_v1_dot_error__pb2
from ...config.v1.ccv.v1 import ccv_config_pb2 as device_dot_heat__pump_dot_config_dot_v1_dot_ccv_dot_v1_dot_ccv__config__pb2
from . import main_pump_flow_pb2 as device_dot_heat__pump_dot_ble_dot_v1_dot_main__pump__flow__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n0device/heat_pump/ble/v1/system_check_state.proto\x12\x17\x64\x65vice.heat_pump.ble.v1\x1a\x1egoogle/protobuf/duration.proto\x1a\x39\x64\x65vice/heat_pump/telemetry/v1/ccv/v1/telemetry_data.proto\x1a%device/heat_pump/state/v1/state.proto\x1a%device/heat_pump/state/v1/error.proto\x1a\x32\x64\x65vice/heat_pump/config/v1/ccv/v1/ccv_config.proto\x1a,device/heat_pump/ble/v1/main_pump_flow.proto\"Z\n\x18GetSystemCheckStateError\x12>\n\x06\x65rrors\x18\x01 \x03(\x0e\x32..device.heat_pump.ble.v1.SystemCheckStateError\"\xa7\x01\n\x1bGetSystemCheckStateResponse\x12:\n\x05state\x18\x01 \x01(\x0b\x32).device.heat_pump.ble.v1.SystemCheckStateH\x00\x12\x42\n\x05\x65rror\x18\x02 \x01(\x0b\x32\x31.device.heat_pump.ble.v1.GetSystemCheckStateErrorH\x00\x42\x08\n\x06group0\"\x83\x0c\n\x10SystemCheckState\x12\x30\n\x06\x65rrors\x18\x02 \x03(\x0b\x32 .device.heat_pump.state.v1.Error\x12I\n\rsensor_values\x18\x03 \x01(\x0b\x32\x32.device.heat_pump.telemetry.v1.ccv.v1.SensorValues\x12\x38\n\x0b\x61ir_purging\x18\x04 \x01(\x0b\x32#.device.heat_pump.ble.v1.AirPurging\x12S\n\x12\x65nergy_calculation\x18\n \x01(\x0b\x32\x37.device.heat_pump.telemetry.v1.ccv.v1.EnergyCalculation\x12!\n\x19system_check_mode_enabled\x18\x0b \x01(\x08\x12W\n\x14\x63\x61lculated_setpoints\x18\x0c \x01(\x0b\x32\x39.device.heat_pump.telemetry.v1.ccv.v1.CalculatedSetpoints\x12\\\n\x17\x63irculation_pump_status\x18\r \x01(\x0b\x32;.device.heat_pump.telemetry.v1.ccv.v1.CirculationPumpStatus\x12V\n\x14inline_heater_status\x18\x0e \x01(\x0b\x32\x38.device.heat_pump.telemetry.v1.ccv.v1.InlineHeaterStatus\x12G\n\x0cvalve_status\x18\x0f \x01(\x0b\x32\x31.device.heat_pump.telemetry.v1.ccv.v1.ValveStatus\x12\x43\n\nsmart_grid\x18\x10 \x01(\x0b\x32/.device.heat_pump.telemetry.v1.ccv.v1.SmartGrid\x12I\n\rmegmet_status\x18\x11 \x01(\x0b\x32\x32.device.heat_pump.telemetry.v1.ccv.v1.MegmetStatus\x12K\n\x0e\x65nergy_balance\x18\x12 \x01(\x0b\x32\x33.device.heat_pump.telemetry.v1.ccv.v1.EnergyBalance\x12G\n\x0cheating_mode\x18\x14 \x01(\x0b\x32\x31.device.heat_pump.telemetry.v1.ccv.v1.HeatingMode\x12\x1b\n\x13manual_mode_enabled\x18\x15 \x01(\x08\x12\x44\n\x10operating_status\x18\x16 \x01(\x0e\x32*.device.heat_pump.state.v1.OperatingStatus\x12H\n\x14main_pump_flow_state\x18\x17 \x01(\x0e\x32*.device.heat_pump.ble.v1.MainPumpFlowState\x12<\n\rforce_heating\x18\x18 \x01(\x0b\x32%.device.heat_pump.ble.v1.ForceHeating\x12\x37\n\x0cstarted_test\x18\x19 \x01(\x0e\x32!.device.heat_pump.ble.v1.TestType\x12\x44\n\x11\x65xternal_hardware\x18\x1a \x01(\x0b\x32).device.heat_pump.ble.v1.ExternalHardware\x12\x44\n\x11\x66low_optimization\x18\x1b \x01(\x0b\x32).device.heat_pump.ble.v1.FlowOptimization\x12\x45\n\x12inline_heater_test\x18\x1c \x01(\x0b\x32).device.heat_pump.ble.v1.InlineHeaterTest\x12K\n\x15\x63ompressor_speed_test\x18\x1d \x01(\x0b\x32,.device.heat_pump.ble.v1.CompressorSpeedTest\"\xe0\x05\n\x13\x43ompressorSpeedTest\x12\r\n\x05speed\x18\x01 \x01(\x05\x12?\n\x04mode\x18\x02 \x01(\x0e\x32\x31.device.heat_pump.ble.v1.CompressorSpeedTest.Mode\x12G\n\x08progress\x18\x03 \x01(\x0e\x32\x35.device.heat_pump.ble.v1.CompressorSpeedTest.Progress\x12*\n\x07runtime\x18\x04 \x01(\x0b\x32\x19.google.protobuf.Duration\x12-\n\ntotal_time\x18\x05 \x01(\x0b\x32\x19.google.protobuf.Duration\x12\x46\n\nlast_error\x18\x06 \x01(\x0e\x32\x32.device.heat_pump.ble.v1.CompressorSpeedTest.Error\"\xe1\x01\n\x05\x45rror\x12\x15\n\x11\x45RROR_UNSPECIFIED\x10\x00\x12\x16\n\x12\x45RROR_FLOW_TOO_LOW\x10\x01\x12\'\n#ERROR_DHW_TANK_TEMPERATURE_TOO_HIGH\x10\x02\x12*\n&ERROR_BUFFER_TANK_TEMPERATURE_TOO_HIGH\x10\x03\x12&\n\"ERROR_PRESSURE_DIFFERENCE_TOO_HIGH\x10\x04\x12\x13\n\x0f\x45RROR_TIMED_OUT\x10\x05\x12\x17\n\x13\x45RROR_TEST_CANCELED\x10\x06\"g\n\x08Progress\x12\x18\n\x14PROGRESS_UNSPECIFIED\x10\x00\x12\x14\n\x10PROGRESS_STOPPED\x10\x01\x12\x15\n\x11PROGRESS_STARTING\x10\x02\x12\x14\n\x10PROGRESS_RUNNING\x10\x03\"@\n\x04Mode\x12\x14\n\x10MODE_UNSPECIFIED\x10\x00\x12\x10\n\x0cMODE_COOLING\x10\x01\x12\x10\n\x0cMODE_HEATING\x10\x02\"\xcf\x04\n\x10InlineHeaterTest\x12\x44\n\x08progress\x18\x01 \x01(\x0e\x32\x32.device.heat_pump.ble.v1.InlineHeaterTest.Progress\x12\x32\n\x0ftotal_test_time\x18\x02 \x01(\x0b\x32\x19.google.protobuf.Duration\x12*\n\x07runtime\x18\x03 \x01(\x0b\x32\x19.google.protobuf.Duration\x12I\n\x0btested_step\x18\x04 \x01(\x0e\x32\x34.device.heat_pump.ble.v1.InlineHeaterTest.TestedStep\"q\n\nTestedStep\x12\x1b\n\x17TESTED_STEP_UNSPECIFIED\x10\x00\x12\x16\n\x12TESTED_STEP_STEP_1\x10\x01\x12\x16\n\x12TESTED_STEP_STEP_2\x10\x02\x12\x16\n\x12TESTED_STEP_STEP_3\x10\x03\"\xd6\x01\n\x08Progress\x12\x18\n\x14PROGRESS_UNSPECIFIED\x10\x00\x12\x17\n\x13PROGRESS_FLOW_CHECK\x10\x01\x12\x14\n\x10PROGRESS_HEATING\x10\x02\x12\x11\n\rPROGRESS_DONE\x10\x03\x12\x1e\n\x1aPROGRESS_FAILED_FLOW_CHECK\x10\x04\x12/\n+PROGRESS_FAILED_CONTROL_BOARD_COMMUNICATION\x10\x05\x12\x1d\n\x19PROGRESS_INVALID_ARGUMENT\x10\x06\"y\n\x10\x46lowOptimization\x12\x16\n\x0emin_pump_speed\x18\x01 \x01(\r\x12M\n\x11outdoor_unit_size\x18\x02 \x01(\x0e\x32\x32.device.heat_pump.config.v1.ccv.v1.OutdoorUnitSize\"R\n\x0c\x46orceHeating\x12\x0f\n\x07\x65nabled\x18\x01 \x01(\x08\x12\x31\n\x0eremaining_time\x18\x02 \x01(\x0b\x32\x19.google.protobuf.Duration\"\x93\x01\n\x10\x45xternalHardware\x12\n\n\x02p1\x18\x01 \x01(\x08\x12\n\n\x02p2\x18\x02 \x01(\x08\x12\x0b\n\x03mv1\x18\x03 \x01(\x08\x12\x0b\n\x03mv2\x18\x04 \x01(\x08\x12M\n\x14inline_heater_config\x18\x05 \x01(\x0b\x32/.device.heat_pump.config.v1.ccv.v1.InlineHeater\"W\n\nAirPurging\x12\x10\n\x08progress\x18\x01 \x01(\r\x12\x37\n\x05state\x18\x02 \x01(\x0e\x32(.device.heat_pump.ble.v1.AirPurgingState*\xc8\x06\n\x15SystemCheckStateError\x12(\n$SYSTEM_CHECK_STATE_ERROR_UNSPECIFIED\x10\x00\x12#\n\x1fSYSTEM_CHECK_STATE_ERROR_ERRORS\x10\x01\x12*\n&SYSTEM_CHECK_STATE_ERROR_SENSOR_VALUES\x10\x02\x12(\n$SYSTEM_CHECK_STATE_ERROR_AIR_PURGING\x10\x03\x12-\n)SYSTEM_CHECK_STATE_ERROR_FLOW_METER_VALUE\x10\x04\x12)\n%SYSTEM_CHECK_STATE_ERROR_AMBIENT_TEMP\x10\x05\x12-\n)SYSTEM_CHECK_STATE_ERROR_INLET_WATER_TEMP\x10\x06\x12.\n*SYSTEM_CHECK_STATE_ERROR_OUTLET_WATER_TEMP\x10\x07\x12,\n(SYSTEM_CHECK_STATE_ERROR_INNER_COIL_TEMP\x10\x08\x12/\n+SYSTEM_CHECK_STATE_ERROR_ENERGY_CALCULATION\x10\t\x12;\n7SYSTEM_CHECK_STATE_ERROR_COLLECTION_ALREADY_IN_PROGRESS\x10\n\x12%\n!SYSTEM_CHECK_STATE_ERROR_INTERNAL\x10\x0b\x12\x31\n-SYSTEM_CHECK_STATE_ERROR_CALCULATED_SETPOINTS\x10\x0c\x12(\n$SYSTEM_CHECK_STATE_ERROR_PUMP_STATUS\x10\r\x12\x31\n-SYSTEM_CHECK_STATE_ERROR_INLINE_HEATER_STATUS\x10\x0e\x12)\n%SYSTEM_CHECK_STATE_ERROR_VALVE_STATUS\x10\x0f\x12\'\n#SYSTEM_CHECK_STATE_ERROR_SMART_GRID\x10\x10\x12*\n&SYSTEM_CHECK_STATE_ERROR_MEGMET_STATUS\x10\x11*\x92\x01\n\x0f\x41irPurgingState\x12!\n\x1d\x41IR_PURGING_STATE_UNSPECIFIED\x10\x00\x12!\n\x1d\x41IR_PURGING_STATE_NOT_STARTED\x10\x01\x12\x1d\n\x19\x41IR_PURGING_STATE_STARTED\x10\x02\x12\x1a\n\x16\x41IR_PURGING_STATE_DONE\x10\x03*\xbf\x02\n\x08TestType\x12\x19\n\x15TEST_TYPE_UNSPECIFIED\x10\x00\x12\x1e\n\x1aTEST_TYPE_AIR_PURGING_HEAT\x10\x01\x12\x17\n\x13TEST_TYPE_HEAT_FLOW\x10\x02\x12\x1d\n\x19TEST_TYPE_AIR_PURGING_DHW\x10\x03\x12\x16\n\x12TEST_TYPE_DHW_FLOW\x10\x04\x12 \n\x1cTEST_TYPE_MANUAL_INTERACTION\x10\x05\x12$\n TEST_TYPE_HEAT_FLOW_OPTIMIZATION\x10\x06\x12#\n\x1fTEST_TYPE_DHW_FLOW_OPTIMIZATION\x10\x07\x12\x1b\n\x17TEST_TYPE_INLINE_HEATER\x10\x08\x12\x1e\n\x1aTEST_TYPE_COMPRESSOR_SPEED\x10\tB\xab\x01\n\x17\x64\x65vice.heat_pump.ble.v1B\x15SystemCheckStateProtoP\x01\xa2\x02\x03\x44HB\xaa\x02\x16\x44\x65vice.HeatPump.Ble.V1\xca\x02\x16\x44\x65vice\\HeatPump\\Ble\\V1\xe2\x02\"Device\\HeatPump\\Ble\\V1\\GPBMetadata\xea\x02\x19\x44\x65vice::HeatPump::Ble::V1b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'device.heat_pump.ble.v1.system_check_state_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\027device.heat_pump.ble.v1B\025SystemCheckStateProtoP\001\242\002\003DHB\252\002\026Device.HeatPump.Ble.V1\312\002\026Device\\HeatPump\\Ble\\V1\342\002\"Device\\HeatPump\\Ble\\V1\\GPBMetadata\352\002\031Device::HeatPump::Ble::V1'
  _globals['_SYSTEMCHECKSTATEERROR']._serialized_start=3928
  _globals['_SYSTEMCHECKSTATEERROR']._serialized_end=4768
  _globals['_AIRPURGINGSTATE']._serialized_start=4771
  _globals['_AIRPURGINGSTATE']._serialized_end=4917
  _globals['_TESTTYPE']._serialized_start=4920
  _globals['_TESTTYPE']._serialized_end=5239
  _globals['_GETSYSTEMCHECKSTATEERROR']._serialized_start=344
  _globals['_GETSYSTEMCHECKSTATEERROR']._serialized_end=434
  _globals['_GETSYSTEMCHECKSTATERESPONSE']._serialized_start=437
  _globals['_GETSYSTEMCHECKSTATERESPONSE']._serialized_end=604
  _globals['_SYSTEMCHECKSTATE']._serialized_start=607
  _globals['_SYSTEMCHECKSTATE']._serialized_end=2146
  _globals['_COMPRESSORSPEEDTEST']._serialized_start=2149
  _globals['_COMPRESSORSPEEDTEST']._serialized_end=2885
  _globals['_COMPRESSORSPEEDTEST_ERROR']._serialized_start=2489
  _globals['_COMPRESSORSPEEDTEST_ERROR']._serialized_end=2714
  _globals['_COMPRESSORSPEEDTEST_PROGRESS']._serialized_start=2716
  _globals['_COMPRESSORSPEEDTEST_PROGRESS']._serialized_end=2819
  _globals['_COMPRESSORSPEEDTEST_MODE']._serialized_start=2821
  _globals['_COMPRESSORSPEEDTEST_MODE']._serialized_end=2885
  _globals['_INLINEHEATERTEST']._serialized_start=2888
  _globals['_INLINEHEATERTEST']._serialized_end=3479
  _globals['_INLINEHEATERTEST_TESTEDSTEP']._serialized_start=3149
  _globals['_INLINEHEATERTEST_TESTEDSTEP']._serialized_end=3262
  _globals['_INLINEHEATERTEST_PROGRESS']._serialized_start=3265
  _globals['_INLINEHEATERTEST_PROGRESS']._serialized_end=3479
  _globals['_FLOWOPTIMIZATION']._serialized_start=3481
  _globals['_FLOWOPTIMIZATION']._serialized_end=3602
  _globals['_FORCEHEATING']._serialized_start=3604
  _globals['_FORCEHEATING']._serialized_end=3686
  _globals['_EXTERNALHARDWARE']._serialized_start=3689
  _globals['_EXTERNALHARDWARE']._serialized_end=3836
  _globals['_AIRPURGING']._serialized_start=3838
  _globals['_AIRPURGING']._serialized_end=3925
# @@protoc_insertion_point(module_scope)


# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: device/heat_pump/cloud/v1/service.proto
# Protobuf Python Version: 6.30.0
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    30,
    0,
    '',
    'device/heat_pump/cloud/v1/service.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from .....util.v2 import market_pb2 as util_dot_v2_dot_market__pb2
from .....util.v1 import uuid_pb2 as util_dot_v1_dot_uuid__pb2
from .....util.v1 import type_of_day_pb2 as util_dot_v1_dot_type__of__day__pb2
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from ...command.v1 import command_progress_pb2 as device_dot_heat__pump_dot_command_dot_v1_dot_command__progress__pb2
from ...command.v1 import command_pb2 as device_dot_heat__pump_dot_command_dot_v1_dot_command__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\'device/heat_pump/cloud/v1/service.proto\x12\x19\x64\x65vice.heat_pump.cloud.v1\x1a\x14util/v2/market.proto\x1a\x12util/v1/uuid.proto\x1a\x19util/v1/type_of_day.proto\x1a\x1fgoogle/protobuf/timestamp.proto\x1a\x32\x64\x65vice/heat_pump/command/v1/command_progress.proto\x1a)device/heat_pump/command/v1/command.proto\"\"\n DeleteLegionellaScheduleResponse\"F\n\x1f\x44\x65leteLegionellaScheduleRequest\x12#\n\x0cheat_pump_id\x18\x01 \x01(\x0b\x32\r.util.v1.Uuid\"w\n\x1dGetLegionellaScheduleResponse\x12#\n\x0cheat_pump_id\x18\x01 \x01(\x0b\x32\r.util.v1.Uuid\x12\x31\n\rnext_run_time\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\"C\n\x1cGetLegionellaScheduleRequest\x12#\n\x0cheat_pump_id\x18\x01 \x01(\x0b\x32\r.util.v1.Uuid\"\x1f\n\x1dSetLegionellaScheduleResponse\"\x97\x01\n\x1cSetLegionellaScheduleRequest\x12#\n\x0cheat_pump_id\x18\x01 \x01(\x0b\x32\r.util.v1.Uuid\x12\x31\n\rnext_run_time\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x1f\n\x06market\x18\x03 \x01(\x0e\x32\x0f.util.v2.Market\"\x1b\n\x19\x44\x65leteDhwScheduleResponse\"?\n\x18\x44\x65leteDhwScheduleRequest\x12#\n\x0cheat_pump_id\x18\x01 \x01(\x0b\x32\r.util.v1.Uuid\"V\n\x16GetDhwScheduleResponse\x12<\n\x07\x65ntries\x18\x01 \x03(\x0b\x32+.device.heat_pump.cloud.v1.DhwScheduleEntry\"<\n\x15GetDhwScheduleRequest\x12#\n\x0cheat_pump_id\x18\x01 \x01(\x0b\x32\r.util.v1.Uuid\"v\n\x10\x44hwScheduleEntry\x12\x12\n\nstart_hour\x18\x01 \x01(\r\x12\x10\n\x08\x65nd_hour\x18\x02 \x01(\r\x12\'\n\x0btype_of_day\x18\x03 \x01(\x0e\x32\x12.util.v1.TypeOfDay\x12\x13\n\x0btemperature\x18\x04 \x01(\r\"\x18\n\x16SetDhwScheduleResponse\"\x9b\x01\n\x15SetDhwScheduleRequest\x12#\n\x0cheat_pump_id\x18\x01 \x01(\x0b\x32\r.util.v1.Uuid\x12<\n\x07\x65ntries\x18\x02 \x03(\x0b\x32+.device.heat_pump.cloud.v1.DhwScheduleEntry\x12\x1f\n\x06market\x18\x03 \x01(\x0e\x32\x0f.util.v2.Market\"!\n\x1f\x44\x65leteScheduledAwayModeResponse\"E\n\x1e\x44\x65leteScheduledAwayModeRequest\x12#\n\x0cheat_pump_id\x18\x01 \x01(\x0b\x32\r.util.v1.Uuid\"i\n\x1cGetScheduledAwayModeResponse\x12I\n\x13scheduled_away_mode\x18\x01 \x01(\x0b\x32,.device.heat_pump.cloud.v1.ScheduledAwayMode\"B\n\x1bGetScheduledAwayModeRequest\x12#\n\x0cheat_pump_id\x18\x01 \x01(\x0b\x32\r.util.v1.Uuid\"I\n\x18ScheduleAwayModeResponse\x12-\n\x16scheduled_away_mode_id\x18\x01 \x01(\x0b\x32\r.util.v1.Uuid\"d\n\x17ScheduleAwayModeRequest\x12I\n\x13scheduled_away_mode\x18\x01 \x01(\x0b\x32,.device.heat_pump.cloud.v1.ScheduledAwayMode\"\xa4\x01\n\x11ScheduledAwayMode\x12#\n\x0cheat_pump_id\x18\x02 \x01(\x0b\x32\r.util.v1.Uuid\x12.\n\nstart_time\x18\x03 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12,\n\x08\x65nd_time\x18\x04 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x0c\n\x04name\x18\x05 \x01(\t\"g\n\x1dStreamCommandProgressResponse\x12\x46\n\x10\x63ommand_progress\x18\x01 \x01(\x0b\x32,.device.heat_pump.command.v1.CommandProgress\"A\n\x1cStreamCommandProgressRequest\x12!\n\ncommand_id\x18\x01 \x01(\x0b\x32\r.util.v1.Uuid\"8\n\x13SendCommandResponse\x12!\n\ncommand_id\x18\x01 \x01(\x0b\x32\r.util.v1.Uuid\"p\n\x12SendCommandRequest\x12#\n\x0cheat_pump_id\x18\x01 \x01(\x0b\x32\r.util.v1.Uuid\x12\x35\n\x07\x63ommand\x18\x02 \x01(\x0b\x32$.device.heat_pump.command.v1.Command2\xcb\x0b\n\x14HeatPumpCloudService\x12~\n\x11\x44\x65leteDhwSchedule\x12\x33.device.heat_pump.cloud.v1.DeleteDhwScheduleRequest\x1a\x34.device.heat_pump.cloud.v1.DeleteDhwScheduleResponse\x12u\n\x0eGetDhwSchedule\x12\x30.device.heat_pump.cloud.v1.GetDhwScheduleRequest\x1a\x31.device.heat_pump.cloud.v1.GetDhwScheduleResponse\x12\x87\x01\n\x14GetScheduledAwayMode\x12\x36.device.heat_pump.cloud.v1.GetScheduledAwayModeRequest\x1a\x37.device.heat_pump.cloud.v1.GetScheduledAwayModeResponse\x12\x90\x01\n\x17\x44\x65leteScheduledAwayMode\x12\x39.device.heat_pump.cloud.v1.DeleteScheduledAwayModeRequest\x1a:.device.heat_pump.cloud.v1.DeleteScheduledAwayModeResponse\x12{\n\x10ScheduleAwayMode\x12\x32.device.heat_pump.cloud.v1.ScheduleAwayModeRequest\x1a\x33.device.heat_pump.cloud.v1.ScheduleAwayModeResponse\x12u\n\x0eSetDhwSchedule\x12\x30.device.heat_pump.cloud.v1.SetDhwScheduleRequest\x1a\x31.device.heat_pump.cloud.v1.SetDhwScheduleResponse\x12\x8a\x01\n\x15GetLegionellaSchedule\x12\x37.device.heat_pump.cloud.v1.GetLegionellaScheduleRequest\x1a\x38.device.heat_pump.cloud.v1.GetLegionellaScheduleResponse\x12\x8a\x01\n\x15SetLegionellaSchedule\x12\x37.device.heat_pump.cloud.v1.SetLegionellaScheduleRequest\x1a\x38.device.heat_pump.cloud.v1.SetLegionellaScheduleResponse\x12\x93\x01\n\x18\x44\x65leteLegionellaSchedule\x12:.device.heat_pump.cloud.v1.DeleteLegionellaScheduleRequest\x1a;.device.heat_pump.cloud.v1.DeleteLegionellaScheduleResponse\x12\x8c\x01\n\x15StreamCommandProgress\x12\x37.device.heat_pump.cloud.v1.StreamCommandProgressRequest\x1a\x38.device.heat_pump.cloud.v1.StreamCommandProgressResponse0\x01\x12l\n\x0bSendCommand\x12-.device.heat_pump.cloud.v1.SendCommandRequest\x1a..device.heat_pump.cloud.v1.SendCommandResponseB\xac\x01\n\x19\x64\x65vice.heat_pump.cloud.v1B\x0cServiceProtoP\x01\xa2\x02\x03\x44HC\xaa\x02\x18\x44\x65vice.HeatPump.Cloud.V1\xca\x02\x18\x44\x65vice\\HeatPump\\Cloud\\V1\xe2\x02$Device\\HeatPump\\Cloud\\V1\\GPBMetadata\xea\x02\x1b\x44\x65vice::HeatPump::Cloud::V1b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'device.heat_pump.cloud.v1.service_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\031device.heat_pump.cloud.v1B\014ServiceProtoP\001\242\002\003DHC\252\002\030Device.HeatPump.Cloud.V1\312\002\030Device\\HeatPump\\Cloud\\V1\342\002$Device\\HeatPump\\Cloud\\V1\\GPBMetadata\352\002\033Device::HeatPump::Cloud::V1'
  _globals['_DELETELEGIONELLASCHEDULERESPONSE']._serialized_start=267
  _globals['_DELETELEGIONELLASCHEDULERESPONSE']._serialized_end=301
  _globals['_DELETELEGIONELLASCHEDULEREQUEST']._serialized_start=303
  _globals['_DELETELEGIONELLASCHEDULEREQUEST']._serialized_end=373
  _globals['_GETLEGIONELLASCHEDULERESPONSE']._serialized_start=375
  _globals['_GETLEGIONELLASCHEDULERESPONSE']._serialized_end=494
  _globals['_GETLEGIONELLASCHEDULEREQUEST']._serialized_start=496
  _globals['_GETLEGIONELLASCHEDULEREQUEST']._serialized_end=563
  _globals['_SETLEGIONELLASCHEDULERESPONSE']._serialized_start=565
  _globals['_SETLEGIONELLASCHEDULERESPONSE']._serialized_end=596
  _globals['_SETLEGIONELLASCHEDULEREQUEST']._serialized_start=599
  _globals['_SETLEGIONELLASCHEDULEREQUEST']._serialized_end=750
  _globals['_DELETEDHWSCHEDULERESPONSE']._serialized_start=752
  _globals['_DELETEDHWSCHEDULERESPONSE']._serialized_end=779
  _globals['_DELETEDHWSCHEDULEREQUEST']._serialized_start=781
  _globals['_DELETEDHWSCHEDULEREQUEST']._serialized_end=844
  _globals['_GETDHWSCHEDULERESPONSE']._serialized_start=846
  _globals['_GETDHWSCHEDULERESPONSE']._serialized_end=932
  _globals['_GETDHWSCHEDULEREQUEST']._serialized_start=934
  _globals['_GETDHWSCHEDULEREQUEST']._serialized_end=994
  _globals['_DHWSCHEDULEENTRY']._serialized_start=996
  _globals['_DHWSCHEDULEENTRY']._serialized_end=1114
  _globals['_SETDHWSCHEDULERESPONSE']._serialized_start=1116
  _globals['_SETDHWSCHEDULERESPONSE']._serialized_end=1140
  _globals['_SETDHWSCHEDULEREQUEST']._serialized_start=1143
  _globals['_SETDHWSCHEDULEREQUEST']._serialized_end=1298
  _globals['_DELETESCHEDULEDAWAYMODERESPONSE']._serialized_start=1300
  _globals['_DELETESCHEDULEDAWAYMODERESPONSE']._serialized_end=1333
  _globals['_DELETESCHEDULEDAWAYMODEREQUEST']._serialized_start=1335
  _globals['_DELETESCHEDULEDAWAYMODEREQUEST']._serialized_end=1404
  _globals['_GETSCHEDULEDAWAYMODERESPONSE']._serialized_start=1406
  _globals['_GETSCHEDULEDAWAYMODERESPONSE']._serialized_end=1511
  _globals['_GETSCHEDULEDAWAYMODEREQUEST']._serialized_start=1513
  _globals['_GETSCHEDULEDAWAYMODEREQUEST']._serialized_end=1579
  _globals['_SCHEDULEAWAYMODERESPONSE']._serialized_start=1581
  _globals['_SCHEDULEAWAYMODERESPONSE']._serialized_end=1654
  _globals['_SCHEDULEAWAYMODEREQUEST']._serialized_start=1656
  _globals['_SCHEDULEAWAYMODEREQUEST']._serialized_end=1756
  _globals['_SCHEDULEDAWAYMODE']._serialized_start=1759
  _globals['_SCHEDULEDAWAYMODE']._serialized_end=1923
  _globals['_STREAMCOMMANDPROGRESSRESPONSE']._serialized_start=1925
  _globals['_STREAMCOMMANDPROGRESSRESPONSE']._serialized_end=2028
  _globals['_STREAMCOMMANDPROGRESSREQUEST']._serialized_start=2030
  _globals['_STREAMCOMMANDPROGRESSREQUEST']._serialized_end=2095
  _globals['_SENDCOMMANDRESPONSE']._serialized_start=2097
  _globals['_SENDCOMMANDRESPONSE']._serialized_end=2153
  _globals['_SENDCOMMANDREQUEST']._serialized_start=2155
  _globals['_SENDCOMMANDREQUEST']._serialized_end=2267
  _globals['_HEATPUMPCLOUDSERVICE']._serialized_start=2270
  _globals['_HEATPUMPCLOUDSERVICE']._serialized_end=3753
# @@protoc_insertion_point(module_scope)


"""BLE interaction class for Aira Home."""
# ble.py
from cryptography.hazmat.primitives.asymmetric import padding
from .device.heat_pump.ble.v1.get_data_pb2 import GetData, DataResponse
from .device.heat_pump.ble.v1.chunked_message_pb2 import ChunkedMessage
from cryptography.x509 import load_pem_x509_certificate
from .utils import Utils, BLEDiscoveryError, BLEConnectionError
from cryptography.hazmat.primitives import hashes
from .util.v1.uuid_pb2 import Uuid as Uuid1
from google.protobuf.message import Message
from bleak import BleakScanner, BleakClient
from pyairahome.enums import GetDataType
from uuid import UUID
from enum import Enum
import asyncio
import os


class Ble:
    """A client to interact with Aira devices over Bluetooth Low Energy (BLE)."""

    def __init__(self, airahome_instance):
        """Initialize Cloud with reference to parent AiraHome instance."""
        self._ah_i = airahome_instance
        self.event_loop = asyncio.get_event_loop()

        # store discovered devices to avoid rescanning
        self._discovery_cache = {} # uuid -> BleDevice
        # create a BleakScanner instance to always use for discovery
        self._scanner = BleakScanner(self._on_device_adv)

        # store parts of received messages to reassemble them afterwards
        self._parts = {}
        self._lengths = {}

        self._client = None

    ###
    # Helper methods
    ###

    def _run_async(self, coro, *args, **kwargs):
        """Helper method to run async methods from sync context."""
        return self.event_loop.run_until_complete(coro(*args, **kwargs))

    def _on_device_adv(self, device, adv_data):
        """Callback for handling device advertisement events."""
        # Check for manufacturer data with company ID 0xFFFF (read below about this)
        if adv_data.manufacturer_data:
            for company_id, data_bytes in adv_data.manufacturer_data.items():
                if company_id == 0xFFFF:
                    try:
                        uuid = str(UUID(data_bytes.hex()))
                    except:
                        uuid = None
                    if uuid:
                        self._discovery_cache[uuid] = device
                        return

    def _on_disconnect(self, client: BleakClient):
        """Callback for handling disconnection events."""
        #self._ah_i.ble.uuid = None
        #self._ah_i.ble_certificate = None
        self._client = None

    def _on_notify(self, sender: int, data: bytearray):
        """Callback for handling notifications from the BLE device."""
        try:
            chunk = ChunkedMessage()
            chunk.ParseFromString(data)

            message_id = chunk.message_id.value.hex() # messages ids are stored as normal hex data
            if message_id not in self._parts:
                self._parts[message_id] = []
                self._lengths[message_id] = chunk.total_bytes
            self._parts[message_id].append(chunk.content)
        except:
            pass # NOT A CHUNKED MESSAGE

    def _rsa_encrypt(self, input_bytes: bytes) -> bytes:
        if not self._ah_i._cert:
            raise ValueError("No certificate loaded for encryption.")
        
        public_key = self._ah_i._cert.public_key()

        # Encrypt the input bytes using RSA with PKCS1 OAEP padding
        ciphertext = public_key.encrypt(
            input_bytes,
            padding.OAEP(
                mgf=padding.MGF1(algorithm=hashes.SHA256()),
                algorithm=hashes.SHA256(),
                label=None
            )
        )
        return ciphertext

    def _send_ble(self, characteristic: str, message: Message, encrypt: bool = False):
        """Send a protobuf message to the connected BLE device. Splits the message into chunks if it exceeds the MTU size. If `encrypt` is True, the message will be encrypted with the certificate key."""
        if not self.is_connected():
            raise BLEConnectionError("Not connected to any BLE device.")
        
        data = message.SerializeToString()
        payload_size = self._ah_i.max_ble_chunk_size

        # chunk the message if it exceeds the payload size or if it must be encrypted, since encrypting will increase the size to 256 bytes
        if len(data) > payload_size or encrypt:
            chunks = [data[i:i + payload_size] for i in range(0, len(data), payload_size)]
            message_id = getattr(data, 'message_id', os.urandom(16)) # if the message has a message_id use it, otherwise generate a random one
            for i,chunk in enumerate(chunks):
                content = chunk if not encrypt else self._rsa_encrypt(chunk)
                chunked_message = ChunkedMessage(
                    message_id=Uuid1(value=message_id),
                    byte_offset=i * payload_size,
                    total_bytes=len(content),
                    content=content
                )
                chunk_data = chunked_message.SerializeToString()
                self._run_async(self._client.write_gatt_char, char_specifier=characteristic, data=chunk_data)
        
        self._run_async(self._client.write_gatt_char, char_specifier=characteristic, data=data) # type: ignore

    def _wait_for_response(self, message_id: Uuid1, timeout: int = -1) -> bytes:
        if timeout < 0:
            timeout = self._ah_i.ble_notify_timeout

        reconstructed = None
        for i in range(timeout * 10):
            self._run_async(asyncio.sleep, 0.1) # sleep

            # check if the sum of part lenghts equals the actual total
            if not self._lengths.get(message_id.value.hex(), False):
                continue

            parts = self._parts.get(message_id.value.hex(), [])
            if sum([len(part) for part in parts]) >= self._lengths[message_id.value.hex()]:
                # all parts received
                reconstructed = b''.join([part for part in parts])
                del self._parts[message_id.value.hex()]
                del self._lengths[message_id.value.hex()]
                break
        if not reconstructed:
            raise TimeoutError(f"No response received for message ID {message_id.value.hex()} within {timeout} seconds.")
        
        return reconstructed

    def add_certificate(self, certificate: str) -> bool:
        """
        Add the aira certificate to allow secure ble communication for commands that require it.

        ### Parameters

        `certificate` : str
            The aira certificate to be added.

        ### Returns
        bool
            True if the certificate was added successfully, False otherwise. Usually will be false if not connected to a device or an invalid certificate is provided.

        ### Examples
        >>> certificate = \"\"\"-----BEGIN CERTIFICATE-----...-----END CERTIFICATE-----\"\"\"
        >>> AiraHome().ble.add_certificate(certificate)
        """
        if not self._ah_i.ble.uuid:
            # No device connected. Please connect to a device before adding a certificate.
            return False
        try:
            self._ah_i.ble_certificate = load_pem_x509_certificate(certificate.encode())
            return True
        except Exception as e:
            return False

    ###
    # Connection methods
    ###

    def is_connected(self) -> bool:
        """
        Check if there is an active BLE connection to a device.

        ### Returns

        bool
            True if connected to a device, False otherwise.

        ### Examples

        >>> AiraHome().ble.is_connected()
        """
        # return False and deletes the client if not connected
        if not self._client:
            return False
        if not self._client.is_connected:
            self._client = None
            return False
        return True

    def discover(self, timeout: int = 5, raw: bool = False) -> dict:
        """
        Returns the list of devices that could be an aira heatpump. NOTICE: Aira is not member of the SIG, therefor it uses company id 0xFFFF which is reserved for development and testing. This means that some discovered devices might not be an actual heatpump.

        ### Parameters

        `timeout` : int, optional
            Timeout for the bluetooth discovery scan in seconds. Defaults to 5 seconds.

        `raw` : bool, optional
            If True, returns the raw discovery cache with BleakDevice objects. Defaults to False.
        ### Returns

        list 
            A list contaning discovered devices as dictionaries with their name and address or the raw BleakDevice objects if `raw=True`.

            Example:
            ```
[{'123e4567-e89b-12d3-a456-426614174000': ('AH-123', '12:34:56:78:9A:BC')}]
            ```
        
        ### Examples

        >>> AiraHome().ble.discover(timeout=5, raw=False)
        """

        found_devices = {} # uuid -> (name, address)
        # Discover devices and advertisement data
        self._discovery_cache = {} # reset cache

        # Start scanning
        self._run_async(self._scanner.start)

        self._run_async(asyncio.sleep, timeout) # sleep to allow devices to be discovered
        
        # Stop scanning and process results
        self._run_async(self._scanner.stop)
        if raw:
            return self._discovery_cache
        
        for uuid, device in self._discovery_cache.items():
            found_devices[uuid] = (device.name, device.address)
        return found_devices

    def connect_uuid(self, uuid: str ) -> bool:
        """
        Connect to a device using its UUID.

        ### Parameters

        `uuid` : str
            The UUID of the device to connect to.

        ### Returns
        bool
            True if the connection was successful, False otherwise. Usually will be false if the device is not found or cannot be connected to.

        ### Examples
        >>> AiraHome().ble.connect_uuid("123e4567-e89b-12d3-a456-426614174000")
        """
        device = self._discovery_cache.get(uuid, None)
        if not device:
            devices = self.discover(timeout=5, raw=True)
            device = devices.get(uuid, None)
            if not device:
                raise BLEDiscoveryError(f"Device with UUID {uuid} not found during discovery. To check if the device is close enough, use discover method.")
        
        self._client = BleakClient(device, disconnected_callback=self._on_disconnect)
        if not self._client:
            raise BLEConnectionError(f"Could not create BLE client for device with UUID {uuid} at address {device.address}.")
        try:
            self._run_async(self._client.connect)
            if self._client.is_connected:
                # Subscribe to notifications on both characteristics
                self._run_async(self._client.start_notify, char_specifier=self._ah_i.insecure_characteristic, callback=self._on_notify)
                self._run_async(self._client.start_notify, char_specifier=self._ah_i.secure_characteristic, callback=self._on_notify)
                return True
            else:
                self._client = None
                raise BLEConnectionError(f"Could not connect to device with UUID {uuid} at address {device.address}.")
        except Exception as e:
            self._client = None
            raise BLEConnectionError(f"Could not connect to device with UUID {uuid} at address {device.address}. Exception: {e}")

    def connect(self) -> bool:
        """Connect to the device using the cloud defined uuid."""
        if not self._ah_i.uuid:
            raise BLEConnectionError("UUID not set. Please set it before running the automatic connection method.")
        return self.connect_uuid(self._ah_i.uuid)

    ###
    # Heatpump methods
    ###

    def get_data(self, data_type: Enum | int, raw: bool = False) -> dict | Message:
        """
        Sends a GetData request to the connected device and returns the complete response.
        Use `raw=True` to get the raw gRPC response.

        ### Parameters

        `data_type` : Enum | int
            The type of data to request, this can be state, system check state, flow data, wifi networks, configuration, power installation. Use pyairahome.enums.GetDataType.* for values.

        `raw` : bool, optional
            If True, returns the raw gRPC response. Defaults to False.

        ### Returns
    
        dict | DataResponse (Message)
            When `raw=False`: A dictionary containing requested data.
            When `raw=True`: The raw gRPC DataResponse protobuf message.
        
        ### Examples

        >>> from pyairahome.enums import GetDataType
        >>> AiraHome().ble.get_data(data_type=Granularity.DATA_TYPE_STATE, raw=False)
        """
        message_id = Uuid1(value=os.urandom(16))
        request = GetData(message_id=message_id,
                          data_type=getattr(data_type, 'value', data_type))
        
        self._send_ble(self._ah_i.insecure_characteristic, request, False)
        response_bytes = self._wait_for_response(message_id)

        # parse the response
        response = DataResponse()
        response.ParseFromString(response_bytes)

        if raw:
            return response
        
        return Utils.convert_to_dict(response)

    def get_states(self, raw: bool = False) -> dict | Message:
        """
        Returns the states of the connected device.
        Use `raw=True` to get the raw gRPC response.

        ### Parameters

        `raw` : bool, optional
            If True, returns the raw gRPC response. Defaults to False.

        ### Returns

        dict | State (Message)
            When `raw=False`: A dictionary containing most states for the given device.
            When `raw=True`: The raw gRPC GetDevicesResponse protobuf message.
            
            Example of the expected response content regardless of the `raw` parameter:
            ```
{'state': [{'allowed_pump_mode_state': 'PUMP_MODE_STATE_IDLE',
                       'aws_iot_received_time': datetime.datetime(2025, 9, 23, 7, 17, 57, 122747),
                       'configured_pump_modes': 'PUMP_MODE_STATE_HEATING_COOLING',
                       'cool_curve_deltas': {},
                       'cool_curves': {...},
                       'current_hot_water_temperature': 23.4,
                       'current_outdoor_temperature': 22.5,
                       'current_pump_mode_state': {...}
                       ...]}
            ```
        
        ### Examples

        >>> AiraHome().ble.get_states(raw=False)
        """
        
        return self.get_data(data_type=GetDataType.DATA_TYPE_STATE, raw=raw)

    def get_system_check_state(self, raw: bool = False) -> dict | Message:
        """
        Returns the system check state of the connected device.
        Use `raw=True` to get the raw gRPC response.

        ### Parameters

        `raw` : bool, optional
            If True, returns the raw gRPC response. Defaults to False.

        ### Returns

        dict | SystemCheckState (Message)
            When `raw=False`: A dictionary containing the system check state for the given device.
            When `raw=True`: The raw gRPC SystemCheckState protobuf message.
        
        Example of the expected response content regardless of the `raw` parameter:
            ```
{'system_check_state': {'air_purging': {'state': 'AIR_PURGING_STATE_NOT_STARTED'},
                        'calculated_setpoints': {...},
                        'circulation_pump_status': {},
                        'compressor_speed_test': {'progress': 'PROGRESS_STOPPED'},
                        'energy_balance': {'energy_balance': 2},
                        'energy_calculation': {'current_electrical_power_w': 19,
                                               'current_phase0': 0.3,
                                               ...
                                               'electrical_energy_cum_kwh': 165,
                                               'electrical_energy_cum_wh': 165900,
                                               'voltage_phase0': 242.7,
                                               ...
                                               'water_energy_cum_kwh': 521,
                                               'water_energy_cum_wh': 521830},
                        ...}}
            ```

        ### Examples

        >>> AiraHome().ble.get_system_check_state(raw=False)
        """

        return self.get_data(data_type=GetDataType.DATA_TYPE_SYSTEM_CHECK_STATE, raw=raw)

    def get_flow_data(self, raw: bool = False) -> dict | Message:
        """
        Returns the flow data of the connected device.
        Use `raw=True` to get the raw gRPC response.

        ### Parameters

        `raw` : bool, optional
            If True, returns the raw gRPC response. Defaults to False.

        ### Returns

        dict | FlowData (Message)
            When `raw=False`: A dictionary containing the flow data for the given device.
            When `raw=True`: The raw gRPC FlowData protobuf message.

        Example of the expected response content regardless of the `raw` parameter:
            ```
{'flow_data': {}} # TODO add example response
            ```

        ### Examples

        >>> AiraHome().ble.get_flow_data(raw=False)
        """

        return self.get_data(data_type=GetDataType.DATA_TYPE_FLOW_DATA, raw=raw)

    def get_wifi_networks(self, raw: bool = False) -> dict | Message:
        """
        Returns the wifi networks close to the connected device.
        Use `raw=True` to get the raw gRPC response.

        ### Parameters

        `raw` : bool, optional
            If True, returns the raw gRPC response. Defaults to False.

        ### Returns

        dict | WifiNetworks (Message)
            When `raw=False`: A dictionary containing the wifi networks known to the given device.
            When `raw=True`: The raw gRPC WifiNetworks protobuf message.

        Example of the expected response content regardless of the `raw` parameter:
            ```
{'wifi_networks': {'wifi_networks': [{'mac_address': '00:1A:11:FF:AA:01',
                                      'password_required': True,
                                      'signal_strength': -66,
                                      'ssid': 'Wifi-Name-123'},
                                     ...]}}
            ```

        ### Examples

        >>> AiraHome().ble.get_wifi_networks(raw=False)
        """

        return self.get_data(data_type=GetDataType.DATA_TYPE_WIFI_NETWORKS, raw=raw)

    def get_configuration(self, raw: bool = False) -> dict | Message:
        """
        Returns the configuration of the connected device.
        Use `raw=True` to get the raw gRPC response.

        ### Parameters

        `raw` : bool, optional
            If True, returns the raw gRPC response. Defaults to False.

        ### Returns

        dict | CcvConfig (Message)
            When `raw=False`: A dictionary containing the configuration for the given device.
            When `raw=True`: The raw gRPC CcvConfig protobuf message.

        Example of the expected response content regardless of the `raw` parameter:
            ```
{'config': {'alarm_thresholds': {...},
            'away_mode': {'dhw_tank_temperature_change': -20.0,
                          'room_temperature_change': -3.0},
            'compressor_settings': {'compressor_inner_coil_block_time': 180,
                                    'compressor_limit_time': 3.0,
                                    'dhw_diverting_valve_prerun_time': 15,
                                    ...}
            ...}}
            ```

        ### Examples

        >>> AiraHome().ble.get_configuration(raw=False)
        """

        return self.get_data(data_type=GetDataType.DATA_TYPE_CONFIGURATION, raw=raw)


# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: util/v2/locale.proto
# Protobuf Python Version: 6.30.0
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    30,
    0,
    '',
    'util/v2/locale.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x14util/v2/locale.proto\x12\x07util.v2*M\n\x06Locale\x12\x16\n\x12LOCALE_UNSPECIFIED\x10\x00\x12\r\n\tLOCALE_IT\x10\x01\x12\r\n\tLOCALE_DE\x10\x02\x12\r\n\tLOCALE_EN\x10\x03\x42S\n\x07util.v2B\x0bLocaleProtoP\x01\xa2\x02\x03UXX\xaa\x02\x07Util.V2\xca\x02\x07Util\\V2\xe2\x02\x13Util\\V2\\GPBMetadata\xea\x02\x08Util::V2b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'util.v2.locale_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\007util.v2B\013LocaleProtoP\001\242\002\003UXX\252\002\007Util.V2\312\002\007Util\\V2\342\002\023Util\\V2\\GPBMetadata\352\002\010Util::V2'
  _globals['_LOCALE']._serialized_start=33
  _globals['_LOCALE']._serialized_end=110
# @@protoc_insertion_point(module_scope)


# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: device/power/peripheral/photovoltaics/v1/config.proto
# Protobuf Python Version: 6.30.0
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    30,
    0,
    '',
    'device/power/peripheral/photovoltaics/v1/config.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n5device/power/peripheral/photovoltaics/v1/config.proto\x12(device.power.peripheral.photovoltaics.v1\"#\n\x13PhotovoltaicsConfig\x12\x0c\n\x04mode\x18\x01 \x01(\x08\x42\xfb\x01\n(device.power.peripheral.photovoltaics.v1B\x0b\x43onfigProtoP\x01\xa2\x02\x03PPP\xaa\x02(Device.Power.Peripheral.Photovoltaics.V1\xca\x02(Device\\Power\\Peripheral\\Photovoltaics\\V1\xe2\x02\x34\x44\x65vice\\Power\\Peripheral\\Photovoltaics\\V1\\GPBMetadata\xea\x02,Device::Power::Peripheral::Photovoltaics::V1b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'device.power.peripheral.photovoltaics.v1.config_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n(device.power.peripheral.photovoltaics.v1B\013ConfigProtoP\001\242\002\003PPP\252\002(Device.Power.Peripheral.Photovoltaics.V1\312\002(Device\\Power\\Peripheral\\Photovoltaics\\V1\342\0024Device\\Power\\Peripheral\\Photovoltaics\\V1\\GPBMetadata\352\002,Device::Power::Peripheral::Photovoltaics::V1'
  _globals['_PHOTOVOLTAICSCONFIG']._serialized_start=99
  _globals['_PHOTOVOLTAICSCONFIG']._serialized_end=134
# @@protoc_insertion_point(module_scope)


# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: device/heat_pump/config/v1/ccv/v1/heat_curve.proto
# Protobuf Python Version: 6.30.0
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    30,
    0,
    '',
    'device/heat_pump/config/v1/ccv/v1/heat_curve.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n2device/heat_pump/config/v1/ccv/v1/heat_curve.proto\x12!device.heat_pump.config.v1.ccv.v1\"\xf8\x01\n\x18PiecewiseLinearCoolCurve\x12H\n\x02p1\x18\x01 \x01(\x0b\x32<.device.heat_pump.config.v1.ccv.v1.PiecewiseLinearCurvePoint\x12H\n\x02p2\x18\x02 \x01(\x0b\x32<.device.heat_pump.config.v1.ccv.v1.PiecewiseLinearCurvePoint\x12H\n\x02p3\x18\x03 \x01(\x0b\x32<.device.heat_pump.config.v1.ccv.v1.PiecewiseLinearCurvePoint\"\x8c\x03\n\x18PiecewiseLinearHeatCurve\x12H\n\x02p1\x18\x01 \x01(\x0b\x32<.device.heat_pump.config.v1.ccv.v1.PiecewiseLinearCurvePoint\x12H\n\x02p2\x18\x02 \x01(\x0b\x32<.device.heat_pump.config.v1.ccv.v1.PiecewiseLinearCurvePoint\x12H\n\x02p3\x18\x03 \x01(\x0b\x32<.device.heat_pump.config.v1.ccv.v1.PiecewiseLinearCurvePoint\x12H\n\x02p4\x18\x04 \x01(\x0b\x32<.device.heat_pump.config.v1.ccv.v1.PiecewiseLinearCurvePoint\x12H\n\x02p5\x18\x05 \x01(\x0b\x32<.device.heat_pump.config.v1.ccv.v1.PiecewiseLinearCurvePoint\"<\n\x19PiecewiseLinearCurvePoint\x12\x0f\n\x07\x61mbient\x18\x01 \x01(\x02\x12\x0e\n\x06supply\x18\x02 \x01(\x02\x42\xd8\x01\n!device.heat_pump.config.v1.ccv.v1B\x0eHeatCurveProtoP\x01\xa2\x02\x03\x43VC\xaa\x02 Device.HeatPump.Config.V1.Ccv.V1\xca\x02 Device\\HeatPump\\Config\\V1\\Ccv\\V1\xe2\x02,Device\\HeatPump\\Config\\V1\\Ccv\\V1\\GPBMetadata\xea\x02%Device::HeatPump::Config::V1::Ccv::V1b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'device.heat_pump.config.v1.ccv.v1.heat_curve_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n!device.heat_pump.config.v1.ccv.v1B\016HeatCurveProtoP\001\242\002\003CVC\252\002 Device.HeatPump.Config.V1.Ccv.V1\312\002 Device\\HeatPump\\Config\\V1\\Ccv\\V1\342\002,Device\\HeatPump\\Config\\V1\\Ccv\\V1\\GPBMetadata\352\002%Device::HeatPump::Config::V1::Ccv::V1'
  _globals['_PIECEWISELINEARCOOLCURVE']._serialized_start=90
  _globals['_PIECEWISELINEARCOOLCURVE']._serialized_end=338
  _globals['_PIECEWISELINEARHEATCURVE']._serialized_start=341
  _globals['_PIECEWISELINEARHEATCURVE']._serialized_end=737
  _globals['_PIECEWISELINEARCURVEPOINT']._serialized_start=739
  _globals['_PIECEWISELINEARCURVEPOINT']._serialized_end=799
# @@protoc_insertion_point(module_scope)


"""Simple class to manage tokens and their expiration."""
# auth/token_manager.py
from ..utils import TokenError
from pycognito import Cognito


class TokenManager:
    def __init__(self,
                 user_pool_id: str,
                 client_id: str,
                 id_token,
                 access_token,
                 refresh_token):
        """Initialize the TokenManager with user pool ID, client ID, and tokens."""
        self.u = Cognito(user_pool_id, client_id, id_token=id_token, access_token=access_token, refresh_token=refresh_token)
            
    def verify_tokens(self):
        """Verify if the tokens are valid."""
        try:
            self.u.verify_tokens()
            return True
        except Exception as e:
            raise TokenError("Token verification failed") from e
    
    def refresh_tokens(self):
        """Refresh the tokens if they are expired."""
        try:
            self.u.check_token()
            return True
        except Exception as e:
            raise TokenError("Token refresh failed") from e

    def get_id_token(self):
        """Get the ID token after validating it."""
        try:
            self.refresh_tokens()
        except TokenError:
            return None
        return self.u.id_token

    def dict(self):
        """Get a dictionary representation of the tokens."""
        return {
            "id_token": self.get_id_token(),
            "access_token": self.u.access_token,
            "refresh_token": self.u.refresh_token
        }


# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: device/heat_pump/ble/v1/chunked_message.proto
# Protobuf Python Version: 6.30.0
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    30,
    0,
    '',
    'device/heat_pump/ble/v1/chunked_message.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from .....util.v1 import uuid_pb2 as util_dot_v1_dot_uuid__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n-device/heat_pump/ble/v1/chunked_message.proto\x12\x17\x64\x65vice.heat_pump.ble.v1\x1a\x12util/v1/uuid.proto\"n\n\x0e\x43hunkedMessage\x12!\n\nmessage_id\x18\x01 \x01(\x0b\x32\r.util.v1.Uuid\x12\x13\n\x0b\x62yte_offset\x18\x02 \x01(\x04\x12\x13\n\x0btotal_bytes\x18\x03 \x01(\x04\x12\x0f\n\x07\x63ontent\x18\x04 \x01(\x0c\x42\xa9\x01\n\x17\x64\x65vice.heat_pump.ble.v1B\x13\x43hunkedMessageProtoP\x01\xa2\x02\x03\x44HB\xaa\x02\x16\x44\x65vice.HeatPump.Ble.V1\xca\x02\x16\x44\x65vice\\HeatPump\\Ble\\V1\xe2\x02\"Device\\HeatPump\\Ble\\V1\\GPBMetadata\xea\x02\x19\x44\x65vice::HeatPump::Ble::V1b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'device.heat_pump.ble.v1.chunked_message_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\027device.heat_pump.ble.v1B\023ChunkedMessageProtoP\001\242\002\003DHB\252\002\026Device.HeatPump.Ble.V1\312\002\026Device\\HeatPump\\Ble\\V1\342\002\"Device\\HeatPump\\Ble\\V1\\GPBMetadata\352\002\031Device::HeatPump::Ble::V1'
  _globals['_CHUNKEDMESSAGE']._serialized_start=94
  _globals['_CHUNKEDMESSAGE']._serialized_end=204
# @@protoc_insertion_point(module_scope)


# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: util/v1/semantic_version.proto
# Protobuf Python Version: 6.30.0
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    30,
    0,
    '',
    'util/v1/semantic_version.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1eutil/v1/semantic_version.proto\x12\x07util.v1\">\n\x0fSemanticVersion\x12\r\n\x05major\x18\x01 \x01(\r\x12\r\n\x05minor\x18\x02 \x01(\r\x12\r\n\x05patch\x18\x03 \x01(\rB\\\n\x07util.v1B\x14SemanticVersionProtoP\x01\xa2\x02\x03UXX\xaa\x02\x07Util.V1\xca\x02\x07Util\\V1\xe2\x02\x13Util\\V1\\GPBMetadata\xea\x02\x08Util::V1b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'util.v1.semantic_version_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\007util.v1B\024SemanticVersionProtoP\001\242\002\003UXX\252\002\007Util.V1\312\002\007Util\\V1\342\002\023Util\\V1\\GPBMetadata\352\002\010Util::V1'
  _globals['_SEMANTICVERSION']._serialized_start=43
  _globals['_SEMANTICVERSION']._serialized_end=105
# @@protoc_insertion_point(module_scope)


# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: device/heat_pump/command/v1/missing_field_error.proto
# Protobuf Python Version: 6.30.0
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    30,
    0,
    '',
    'device/heat_pump/command/v1/missing_field_error.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n5device/heat_pump/command/v1/missing_field_error.proto\x12\x1b\x64\x65vice.heat_pump.command.v1\"\x0f\n\rZoneSetpoints\"\x0b\n\tTimestamp\"\xb2\x02\n\x11MissingFieldError\x12G\n\x10heat_curve_point\x18\x01 \x01(\x0e\x32+.device.heat_pump.command.v1.HeatCurvePointH\x00\x12;\n\ttimestamp\x18\x02 \x01(\x0b\x32&.device.heat_pump.command.v1.TimestampH\x00\x12\x44\n\x0ezone_setpoints\x18\x03 \x01(\x0b\x32*.device.heat_pump.command.v1.ZoneSetpointsH\x00\x12G\n\x10\x63ool_curve_point\x18\x04 \x01(\x0e\x32+.device.heat_pump.command.v1.CoolCurvePointH\x00\x42\x08\n\x06group0*}\n\x0e\x43oolCurvePoint\x12 \n\x1c\x43OOL_CURVE_POINT_UNSPECIFIED\x10\x00\x12\x17\n\x13\x43OOL_CURVE_POINT_P1\x10\x01\x12\x17\n\x13\x43OOL_CURVE_POINT_P2\x10\x02\x12\x17\n\x13\x43OOL_CURVE_POINT_P3\x10\x03*\xaf\x01\n\x0eHeatCurvePoint\x12 \n\x1cHEAT_CURVE_POINT_UNSPECIFIED\x10\x00\x12\x17\n\x13HEAT_CURVE_POINT_P1\x10\x01\x12\x17\n\x13HEAT_CURVE_POINT_P2\x10\x02\x12\x17\n\x13HEAT_CURVE_POINT_P3\x10\x03\x12\x17\n\x13HEAT_CURVE_POINT_P4\x10\x04\x12\x17\n\x13HEAT_CURVE_POINT_P5\x10\x05\x42\xc0\x01\n\x1b\x64\x65vice.heat_pump.command.v1B\x16MissingFieldErrorProtoP\x01\xa2\x02\x03\x44HC\xaa\x02\x1a\x44\x65vice.HeatPump.Command.V1\xca\x02\x1a\x44\x65vice\\HeatPump\\Command\\V1\xe2\x02&Device\\HeatPump\\Command\\V1\\GPBMetadata\xea\x02\x1d\x44\x65vice::HeatPump::Command::V1b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'device.heat_pump.command.v1.missing_field_error_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\033device.heat_pump.command.v1B\026MissingFieldErrorProtoP\001\242\002\003DHC\252\002\032Device.HeatPump.Command.V1\312\002\032Device\\HeatPump\\Command\\V1\342\002&Device\\HeatPump\\Command\\V1\\GPBMetadata\352\002\035Device::HeatPump::Command::V1'
  _globals['_COOLCURVEPOINT']._serialized_start=425
  _globals['_COOLCURVEPOINT']._serialized_end=550
  _globals['_HEATCURVEPOINT']._serialized_start=553
  _globals['_HEATCURVEPOINT']._serialized_end=728
  _globals['_ZONESETPOINTS']._serialized_start=86
  _globals['_ZONESETPOINTS']._serialized_end=101
  _globals['_TIMESTAMP']._serialized_start=103
  _globals['_TIMESTAMP']._serialized_end=114
  _globals['_MISSINGFIELDERROR']._serialized_start=117
  _globals['_MISSINGFIELDERROR']._serialized_end=423
# @@protoc_insertion_point(module_scope)


# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: device/power/peripheral/battery/v1/config.proto
# Protobuf Python Version: 6.30.0
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    30,
    0,
    '',
    'device/power/peripheral/battery/v1/config.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n/device/power/peripheral/battery/v1/config.proto\x12\"device.power.peripheral.battery.v1\"\x94\x01\n\rBatteryConfig\x12\x1e\n\x16maximum_charging_power\x18\x01 \x01(\x02\x12!\n\x19maximum_discharging_power\x18\x02 \x01(\x02\x12\x1f\n\x17minimum_charge_capacity\x18\x03 \x01(\x02\x12\x1f\n\x17maximum_charge_capacity\x18\x04 \x01(\x02\x42\xdd\x01\n\"device.power.peripheral.battery.v1B\x0b\x43onfigProtoP\x01\xa2\x02\x03PPB\xaa\x02\"Device.Power.Peripheral.Battery.V1\xca\x02\"Device\\Power\\Peripheral\\Battery\\V1\xe2\x02.Device\\Power\\Peripheral\\Battery\\V1\\GPBMetadata\xea\x02&Device::Power::Peripheral::Battery::V1b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'device.power.peripheral.battery.v1.config_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\"device.power.peripheral.battery.v1B\013ConfigProtoP\001\242\002\003PPB\252\002\"Device.Power.Peripheral.Battery.V1\312\002\"Device\\Power\\Peripheral\\Battery\\V1\342\002.Device\\Power\\Peripheral\\Battery\\V1\\GPBMetadata\352\002&Device::Power::Peripheral::Battery::V1'
  _globals['_BATTERYCONFIG']._serialized_start=88
  _globals['_BATTERYCONFIG']._serialized_end=236
# @@protoc_insertion_point(module_scope)